#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

try:
    from django.utils.crypto import salted_hmac
except ImportError:
    raise ImportError("This module depends on django.")


def get_hmac(key_salt, value):
    return salted_hmac(key_salt, value).hexdigest()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.contrib.auth.models import AbstractUser


class User(AbstractUser):
    class Meta(AbstractUser.Meta):
        app_label = "db"
        db_table = "db_user"
        swappable = "AUTH_USER_MODEL"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Optional

USER_ROOT = 1
USER_SYSTEM_ID = -1
USER_SYSTEM_NAME = "Polyaxon"


def is_system_user(user_id: Optional[int] = None) -> bool:
    if user_id:
        return user_id == USER_SYSTEM_ID

    return False



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import models


class VisibilityModel(models.Model):
    is_public = models.BooleanField(
        default=False, help_text="If the entity is public or private."
    )

    class Meta:
        abstract = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("db", "0010_auto_20210429_1539"),
    ]

    operations = [
        migrations.AlterField(
            model_name="artifact",
            name="state",
            field=models.UUIDField(db_index=True),
        ),
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from rest_framework.relations import SlugRelatedField


class UuidSlugRelatedField(SlugRelatedField):
    def to_representation(self, obj):
        value = getattr(obj, self.slug_field)
        if value:
            return value.hex
        return value



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common import conf
from haupt.common.options.registry import core

conf.subscribe(core.Logging)
conf.subscribe(core.Debug)
conf.subscribe(core.Protocol)
conf.subscribe(core.CeleryBrokerBackend)
conf.subscribe(core.CeleryBrokerUrl)
conf.subscribe(core.SecretInternalToken)
conf.subscribe(core.HealthCheckWorkerTimeout)
conf.subscribe(core.SchedulerEnabled)
conf.subscribe(core.UiAdminEnabled)
conf.subscribe(core.UiAssetsVersion)
conf.subscribe(core.UiBaseUrl)
conf.subscribe(core.UiOffline)
conf.subscribe(core.UiEnabled)
conf.subscribe(core.UiInSandbox)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import click


@click.command()
@click.option(
    "--host",
    help="The service host.",
)
@click.option(
    "--port",
    type=int,
    help="The service port.",
)
@click.option(
    "--workers",
    type=int,
    help="Number of workers.",
)
@click.option(
    "--per-core",
    is_flag=True,
    default=False,
    help="To enable workers per core.",
)
@click.option(
    "--uds",
    help="UNIX domain socket binding.",
)
def server(host: str, port: int, workers: int, per_core: bool, uds: str):
    """Start a new sever session."""
    from haupt.cli.runners.server import start

    return start(
        host=host,
        port=port,
        workers=workers,
        per_core=per_core,
        uds=uds,
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.options.option import Option, OptionScope, OptionStores

K8S_NAMESPACE = "K8S_NAMESPACE"
K8S_IN_CLUSTER = "K8S_IN_CLUSTER"

OPTIONS = {K8S_NAMESPACE, K8S_IN_CLUSTER}


class K8SNamespace(Option):
    key = K8S_NAMESPACE
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = False
    is_list = False
    typing = "str"
    store = OptionStores.SETTINGS
    default = None
    options = None


class K8SInCluster(Option):
    key = K8S_IN_CLUSTER
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = False
    is_list = False
    typing = "bool"
    store = OptionStores.SETTINGS
    default = None
    options = None



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import uuid

from django.db import models


class UuidModel(models.Model):
    uuid = models.UUIDField(default=uuid.uuid4, editable=False, unique=True, null=False)

    class Meta:
        abstract = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Optional

from haupt.proxies.schemas.base import clean_config, get_config

UPSTREAM_OPTIONS = """
upstream polyaxon {{
  server unix:{root}/web/polyaxon.sock;
}}
"""

SERVER_OPTIONS = """
server {{
    include polyaxon/polyaxon.base.conf;
}}
"""

REDIRECT_OPTIONS = """
include polyaxon/polyaxon.redirect.conf;
"""


def get_server_config(
    root: Optional[str] = None, use_upstream: bool = False, use_redirect: bool = False
):
    config = []
    if use_upstream:
        root = root or "/polyaxon"
        config.append(get_config(options=UPSTREAM_OPTIONS, indent=0, root=root))

    config.append(get_config(options=SERVER_OPTIONS, indent=0, root=root))
    if use_redirect:
        config.append(get_config(options=REDIRECT_OPTIONS, indent=0))
    return clean_config(config)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.events.event import Attribute

OWNER_ATTRIBUTES = (Attribute("id"), Attribute("name"))

OWNER_RESOURCE_ATTRIBUTES = (
    Attribute("uuid", is_uuid=True),
    Attribute("name"),
    Attribute("owner.name"),
    Attribute("owner_id"),
)

AGENT_RESOURCE_ATTRIBUTES = (
    Attribute("uuid", is_uuid=True),
    Attribute("name"),
    Attribute("owner.name"),
    Attribute("owner_id"),
    Attribute("agent_name"),
    Attribute("agent_uuid"),
    Attribute("agent_id"),
)

SERVICE_ACCOUNT_RESOURCE_ATTRIBUTES = (
    Attribute("uuid", is_uuid=True),
    Attribute("name"),
    Attribute("owner.name"),
    Attribute("owner_id"),
    Attribute("sa_name"),
    Attribute("sa_uuid"),
    Attribute("sa_id"),
)

PROJECT_OWNER_ATTRIBUTES = (
    Attribute("name"),
    Attribute("owner.name"),
    Attribute("owner_id"),
)

PROJECT_RESOURCE_ATTRIBUTES = (
    Attribute("uuid", is_uuid=True),
    Attribute("project.uuid", is_uuid=True),
    Attribute("project.name"),
    Attribute("project.owner.name"),
    Attribute("name", is_required=False),
)

PROJECT_RESOURCE_OWNER_ATTRIBUTES = (
    Attribute("uuid", is_uuid=True),
    Attribute("project.uuid", is_uuid=True),
    Attribute("project.name"),
    Attribute("project.owner.name"),
    Attribute("owner_id"),
    Attribute("name", is_required=False),
)

PROJECT_RUN_EXECUTOR_ATTRIBUTES = (
    Attribute("uuid", is_uuid=True),
    Attribute("project.uuid", is_uuid=True),
    Attribute("project.name"),
    Attribute("project.owner.name"),
    Attribute("name", is_required=False),
    Attribute("is_managed", attr_type=bool, is_required=False),
    Attribute("pipeline_id", attr_type=int, is_required=False),
)

PROJECT_RUN_EXECUTOR_OWNER_ATTRIBUTES = (
    Attribute("uuid", is_uuid=True),
    Attribute("project.uuid", is_uuid=True),
    Attribute("project.name"),
    Attribute("owner_id"),
    Attribute("project.owner.name"),
    Attribute("name", is_required=False),
    Attribute("is_managed", attr_type=bool, is_required=False),
    Attribute("pipeline_id", attr_type=int, is_required=False),
)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from clipped.utils.json import orjson_loads
from pydantic import ValidationError as PydanticValidationError
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response

from haupt.db.managers.runs import base_approve_run
from haupt.db.managers.statuses import new_run_status, new_run_stopping_status
from polyaxon.exceptions import PolyaxonException
from polyaxon.lifecycle import V1StatusCondition


def clone_run(view, request, *args, **kwargs):
    view.run = view.get_object()
    run = view.pre_validate(view.run)
    content = None
    if "content" in request.data:
        content = request.data["content"]
    if content and not isinstance(content, dict):
        try:
            content = orjson_loads(content)
        except Exception as e:
            raise ValidationError("Cloning was not successful, error: {}".format(e))
    try:
        new_obj = view.clone(
            obj=run,
            content=content,
            name=request.data.get("name"),
            description=request.data.get("description"),
            tags=request.data.get("tags"),
            meta_info=request.data.get("meta_info"),
        )
    except (PydanticValidationError, PolyaxonException, ValueError) as e:
        raise ValidationError("Cloning was not successful, error: {}".format(e))

    view.audit(request, *args, **kwargs)
    serializer = view.get_serializer(new_obj)
    return Response(status=status.HTTP_201_CREATED, data=serializer.data)


def create_status(view, serializer):
    serializer.is_valid()
    validated_data = serializer.validated_data
    if not validated_data:
        return
    condition = None
    if validated_data.get("condition"):
        condition = V1StatusCondition.get_condition(**validated_data.get("condition"))
    if condition:
        new_run_status(
            run=view.run, condition=condition, force=validated_data.get("force", False)
        )


def stop_run(view, request, *args, **kwargs):
    if new_run_stopping_status(run=view.run, message="User requested to stop the run."):
        view.audit(request, *args, **kwargs)
    return Response(status=status.HTTP_200_OK, data={})


def approve_run(view, request, *args, **kwargs):
    pending = view.run.pending
    if pending:
        base_approve_run(view.run)
        view.audit(request, *args, **kwargs)
    return Response(status=status.HTTP_200_OK, data={})



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.db.administration.utils import DiffModelAdmin


class ProjectAdmin(DiffModelAdmin):
    list_display = ("uuid", "name", "created_at", "updated_at")
    list_display_links = ("uuid", "name")
    readonly_fields = DiffModelAdmin.readonly_fields + ("name",)
    fields = ("name", "live_state", "created_at", "updated_at")

    def get_queryset(self, request):
        qs = self.model.all.get_queryset()
        ordering = self.get_ordering(request)
        if ordering:
            qs = qs.order_by(*ordering)
        return qs



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from django.conf import settings

from haupt.db.query_managers import callback_conditions
from haupt.db.query_managers.manager import BaseQueryManager
from polyaxon.pql.builder import (
    ArrayCondition,
    BoolCondition,
    CallbackCondition,
    ComparisonCondition,
    DateTimeCondition,
    KeysCondition,
    SearchCondition,
    ValueCondition,
)
from polyaxon.pql.parser import (
    parse_cpu_operation,
    parse_datetime_operation,
    parse_memory_operation,
    parse_scalar_operation,
    parse_search_operation,
    parse_value_operation,
)


class RunQueryManager(BaseQueryManager):
    NAME = "run"
    FIELDS_USE_NAME = {
        "project",
        "agent",
        "queue",
        "artifacts_store",
        "connections",
    }
    FIELDS_USE_STATE = {"artifacts"}
    FIELDS_USE_UUID = {
        "original",
        "upstream",
        "downstream",
        "pipeline",
        "controller",
        "upstream_runs",
        "downstream_runs",
    }
    FIELDS_PROXY = {
        "params": "inputs",
        "in": "inputs",
        "out": "outputs",
        "metrics": "outputs",
        "meta_values": "meta_info",
        "meta_flags": "meta_info",
        "id": "uuid",
        "uid": "uuid",
        "upstream": "upstream_runs",
        "downstream": "downstream_runs",
        "user": "user__username",
    }
    FIELDS_DISTINCT = {
        "artifacts",
        "connections",
    }
    FIELDS_ORDERING = (
        "created_at",
        "updated_at",
        "started_at",
        "finished_at",
        "schedule_at",
        "name",
        "kind",
        "namespace",
        "runtime",
        "user",
        "uuid",
        "duration",
        "wait_time",
        "status",
        "cost",
        "cpu",
        "memory",
        "gpu",
        "custom",
        "state",
        "component_state",
    )
    FIELDS_ORDERING_PROXY = {
        "metrics": {"field": "outputs", "annotate": True},
        "params": {"field": "inputs", "annotate": True},
        "inputs": {"field": "inputs", "annotate": True},
        "in": {"field": "inputs", "annotate": True},
        "outputs": {"field": "outputs", "annotate": True},
        "out": {"field": "outputs", "annotate": True},
        "meta_flags": {"field": "meta_info", "annotate": True},
        "meta_info": {"field": "meta_info", "annotate": True},
        "meta_values": {"field": "meta_info", "annotate": True},
    }
    FIELDS_DEFAULT_ORDERING = ("-updated_at",)
    CHECK_ALIVE = True
    PARSERS_BY_FIELD = {
        # Uuid
        "id": parse_search_operation,
        "uid": parse_search_operation,
        "uuid": parse_search_operation,
        # Dates
        "created_at": parse_datetime_operation,
        "updated_at": parse_datetime_operation,
        "started_at": parse_datetime_operation,
        "finished_at": parse_datetime_operation,
        "schedule_at": parse_datetime_operation,
        # Name
        "name": parse_search_operation,
        # Description
        "description": parse_search_operation,
        # User
        "user": parse_value_operation,
        # Status
        "status": parse_value_operation,
        # Project
        "project": parse_value_operation,
        # Original
        "original": parse_value_operation,
        # Pipeline
        "pipeline": parse_value_operation,
        # Controller
        "controller": parse_value_operation,
        # Upstream
        "upstream": parse_value_operation,
        # Downstream
        "downstream": parse_value_operation,
        # Cloning kind
        "cloning_kind": parse_value_operation,
        # Artifact
        "in_artifact_kind": parse_value_operation,
        "out_artifact_kind": parse_value_operation,
        # Backend
        "backend": parse_value_operation,
        # Framework
        "framework": parse_value_operation,
        # Commit
        "commit": parse_value_operation,
        # Kind
        "kind": parse_value_operation,
        # Meta Kind
        "runtime": parse_value_operation,
        # Namespace
        "namespace": parse_value_operation,
        # Params
        "params": parse_value_operation,
        "inputs": parse_value_operation,
        "in": parse_value_operation,
        # Results
        "outputs": parse_value_operation,
        "out": parse_value_operation,
        # Metrics
        "metrics": parse_scalar_operation,
        # Meta
        "meta_flags": parse_value_operation,
        "meta_info": parse_value_operation,
        "meta_values": parse_scalar_operation,
        # Tags
        "tags": parse_value_operation,
        # Live state
        "live_state": parse_value_operation,
        # Duration
        "duration": parse_scalar_operation,
        # Wait time
        "wait_time": parse_scalar_operation,
        # Agent
        "agent": parse_value_operation,
        "queue": parse_value_operation,
        # Artifacts store
        "artifacts_store": parse_value_operation,
        # States
        "state": parse_value_operation,
        "component_state": parse_value_operation,
        # Flags
        "is_managed": parse_value_operation,
        "pending": parse_value_operation,
        # Resources
        "cost": parse_scalar_operation,
        "cpu": parse_cpu_operation,
        "memory": parse_memory_operation,
        "gpu": parse_scalar_operation,
        "custom": parse_scalar_operation,
        # Artifacts
        "artifacts": parse_value_operation,
        # Connections
        "connections": parse_value_operation,
    }
    CONDITIONS_BY_FIELD = {
        # Uuid
        "id": SearchCondition,
        "uid": SearchCondition,
        "uuid": SearchCondition,
        # Dates
        "created_at": DateTimeCondition,
        "updated_at": DateTimeCondition,
        "started_at": DateTimeCondition,
        "finished_at": DateTimeCondition,
        "schedule_at": DateTimeCondition,
        # Name
        "name": SearchCondition,
        # Description
        "description": SearchCondition,
        # User
        "user": ValueCondition,
        # Status
        "status": ValueCondition,
        # Project
        "project": ValueCondition,
        # Original
        "original": ValueCondition,
        # Pipeline
        "pipeline": ValueCondition,
        # Controller
        "controller": ValueCondition,
        # Upstream
        "upstream": ValueCondition,
        # Downstream
        "downstream": ValueCondition,
        # Cloning kind
        "cloning_kind": ValueCondition,
        # Artifact
        "in_artifact_kind": CallbackCondition(
            callback_conditions.in_artifact_kind_condition
        ),
        "out_artifact_kind": CallbackCondition(
            callback_conditions.in_artifact_kind_condition
        ),
        # Backend
        "backend": ValueCondition,
        # Framework
        "framework": ValueCondition,
        # Commit
        "commit": CallbackCondition(callback_conditions.commit_condition),
        # Kind
        "kind": ValueCondition,
        # Meta Kind
        "runtime": ValueCondition,
        # Namespace
        "namespace": ValueCondition,
        # Params
        "params": ComparisonCondition,
        "inputs": ComparisonCondition,
        "in": ComparisonCondition,
        # Results
        "outputs": ComparisonCondition,
        "out": ComparisonCondition,
        # Metrics
        "metrics": ComparisonCondition,
        # Meta
        "meta_flags": BoolCondition,
        "meta_info": ValueCondition,
        "meta_values": ValueCondition,
        # Tags
        "tags": KeysCondition
        if settings.DB_ENGINE_NAME == "sqlite"
        else ArrayCondition,
        # Live state
        "live_state": ValueCondition,
        # Duration
        "duration": ComparisonCondition,
        # Wait time
        "wait_time": ComparisonCondition,
        # Agent
        "agent": ValueCondition,
        "queue": ValueCondition,
        # Artifacts store
        "artifacts_store": ValueCondition,
        # States
        "state": ValueCondition,
        "component_state": ValueCondition,
        # Flags
        "is_managed": BoolCondition,
        "pending": ValueCondition,
        # Resources
        "cost": ComparisonCondition,
        "cpu": ComparisonCondition,
        "memory": ComparisonCondition,
        "gpu": ComparisonCondition,
        "custom": ComparisonCondition,
        # Artifacts
        "artifacts": ValueCondition,
        # Connections
        "connections": ValueCondition,
    }



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from clipped.formatting import Printer
from pydantic import ValidationError

from polyaxon.services.values import PolyaxonServices

PROXIES_CONFIG = None
SANDBOX_CONFIG = None

PolyaxonServices.set_service_name()


def set_proxies_config():
    from haupt.managers.proxies import ProxiesManager

    global PROXIES_CONFIG

    PROXIES_CONFIG = ProxiesManager.get_config_from_env()


def set_sandbox_config():
    from haupt.managers.sandbox import SandboxConfigManager
    from polyaxon.contexts.paths import mount_sandbox
    from polyaxon.settings import HOME_CONFIG, set_agent_config

    SandboxConfigManager.set_config_path(HOME_CONFIG.path)
    mount_sandbox()
    PolyaxonServices.set_service_name(PolyaxonServices.SANDBOX)

    global SANDBOX_CONFIG

    try:
        SANDBOX_CONFIG = SandboxConfigManager.get_config_or_default()
        set_agent_config(SANDBOX_CONFIG)
    except (TypeError, ValidationError):
        SandboxConfigManager.purge()
        Printer.warning("Your sandbox configuration was purged!")



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.background.celeryp.tasks import CoreSchedulerCeleryTasks
from haupt.common import conf
from haupt.common.options.registry.core import SCHEDULER_ENABLED
from haupt.orchestration.scheduler import manager
from polyaxon.constants.metadata import META_EAGER_MODE
from polyaxon.lifecycle import V1Statuses


def handle_run_created(workers_backend, event: "Event") -> None:  # noqa: F821
    """Handles creation, resume, and restart"""
    eager = False
    if (
        event.instance
        and event.instance.status != V1Statuses.RESUMING
        and (event.instance.meta_info or {}).get(META_EAGER_MODE)
    ):
        eager = True
    if not eager:
        eager = (
            not event.data["is_managed"]
            and event.instance
            and event.instance.content is not None
        )
    # Run is not managed by Polyaxon
    if not event.data["is_managed"] and not eager:
        return
    # Run is managed by a pipeline
    if event.data.get("pipeline_id") is not None:
        return
    # Run is pending
    if event.instance.pending is not None:
        return

    if conf.get(SCHEDULER_ENABLED) and not eager:
        workers_backend.send(
            CoreSchedulerCeleryTasks.RUNS_PREPARE, kwargs={"run_id": event.instance_id}
        )
        return

    # Eager mode
    manager.runs_prepare(run_id=event.instance_id, run=event.instance, eager=True)


def handle_run_approved_triggered(
    workers_backend, event: "Event"
) -> None:  # noqa: F821
    run = manager.get_run(run_id=event.instance_id, run=event.instance)
    if not run:
        return

    # Check if it should prepare
    if run.status == V1Statuses.CREATED:
        if conf.get(SCHEDULER_ENABLED):
            workers_backend.send(
                CoreSchedulerCeleryTasks.RUNS_PREPARE,
                kwargs={"run_id": event.instance_id},
            )
            return

        # Eager mode
        manager.runs_prepare(run_id=event.instance_id, run=event.instance, eager=True)

    # Should start
    if run.is_managed and conf.get(SCHEDULER_ENABLED):
        workers_backend.send(
            CoreSchedulerCeleryTasks.RUNS_START, kwargs={"run_id": event.instance_id}
        )
        return

    manager.runs_start(run_id=event.instance_id, run=event.instance)


def handle_run_stopped_triggered(workers_backend, event: "Event") -> None:  # noqa: F821
    run = manager.get_run(run_id=event.instance_id, run=event.instance)
    if not run:
        return

    if run.is_managed and conf.get(SCHEDULER_ENABLED):
        workers_backend.send(
            CoreSchedulerCeleryTasks.RUNS_STOP, kwargs={"run_id": event.instance_id}
        )
        return

    manager.runs_stop(run_id=event.instance_id, run=event.instance)


def handle_new_artifacts(workers_backend, event: "Event") -> None:  # noqa: F821
    artifacts = event.data.get("artifacts")
    if not artifacts:
        return

    if conf.get(SCHEDULER_ENABLED):
        workers_backend.send(
            CoreSchedulerCeleryTasks.RUNS_SET_ARTIFACTS,
            kwargs={"run_id": event.instance_id, "artifacts": artifacts},
        )
        return

    manager.runs_set_artifacts(
        run_id=event.instance_id, run=event.instance, artifacts=artifacts
    )


def handle_run_deleted(workers_backend, event: "Event") -> None:  # noqa: F821
    run = manager.get_run(run_id=event.instance_id, run=event.instance)
    if not run:
        return

    if not run.is_managed:
        run.delete()
        return

    if conf.get(SCHEDULER_ENABLED):
        run.delete_in_progress()
        workers_backend.send(
            CoreSchedulerCeleryTasks.RUNS_DELETE, kwargs={"run_id": run.id}
        )
    else:
        manager.runs_delete(run_id=run.id, run=run)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework import fields, serializers
from rest_framework.exceptions import ValidationError

from django.db import IntegrityError

from haupt.apis.serializers.base.tags import TagsMixin
from haupt.db.abstracts.getter import get_project_model


class ProjectNameSerializer(serializers.ModelSerializer):
    class Meta:
        model = get_project_model()
        fields = ("name",)


class ProjectSerializer(TagsMixin, serializers.ModelSerializer):
    uuid = fields.UUIDField(format="hex", read_only=True)

    class Meta:
        model = get_project_model()
        fields = (
            "uuid",
            "name",
            "description",
            "tags",
            "created_at",
            "updated_at",
        )


class ProjectDetailSerializer(ProjectSerializer):
    class Meta(ProjectSerializer.Meta):
        fields = ProjectSerializer.Meta.fields + (
            "readme",
            "live_state",
        )

    def update(self, instance, validated_data):
        validated_data = self.validated_tags(
            validated_data=validated_data, tags=instance.tags
        )

        try:
            return super().update(instance=instance, validated_data=validated_data)
        except IntegrityError:
            raise ValidationError(
                f"A project with name {validated_data['name']} already exists."
            )


class ProjectCreateSerializer(ProjectSerializer):
    class Meta(ProjectSerializer.Meta):
        fields = ProjectSerializer.Meta.fields + ("readme",)

    def create(self, validated_data):
        try:
            return super().create(validated_data)
        except IntegrityError:
            raise ValidationError(
                f"A project with name {validated_data['name']} already exists."
            )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework.urlpatterns import format_suffix_patterns

from django.urls import re_path

from haupt.apis.versions import views
from haupt.common.apis.urls import versions

urlpatterns = [
    re_path(versions.URLS_VERSIONS_INSTALLED, views.VersionsInstalledView.as_view()),
    re_path(
        versions.URLS_VERSIONS_COMPATIBILITY,
        views.VersionsCompatibilityView.as_view(),
    ),
]

urlpatterns = format_suffix_patterns(urlpatterns)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.options import option_namespaces, option_subjects
from haupt.common.options.option import Option, OptionScope, OptionStores
from polyaxon.auxiliaries import (
    V1PolyaxonInitContainer,
    V1PolyaxonSidecarContainer,
    get_default_init_container,
    get_default_sidecar_container,
)

INIT_CONTAINER = "{}_{}".format(option_namespaces.INIT, option_subjects.CONTAINER)

SIDECAR_CONTAINER = "{}_{}".format(
    option_namespaces.SIDECARS, option_subjects.CONTAINER
)

OPTIONS = {SIDECAR_CONTAINER, INIT_CONTAINER}


class PolyaxonInitContainer(Option):
    key = INIT_CONTAINER
    description = "The docker image to use for init container"
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "str"
    store = OptionStores.SETTINGS
    options = None

    @staticmethod
    def get_default_value():
        return get_default_init_container(schema=False)

    @classmethod
    def _extra_processing(cls, value):
        if not value:
            return value
        V1PolyaxonInitContainer.from_dict(value)
        return value


class PolyaxonSidecarContainer(Option):
    key = SIDECAR_CONTAINER
    description = "Sidecar container definition"
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    typing = "dict"
    store = OptionStores.SETTINGS
    options = None

    @staticmethod
    def get_default_value():
        return get_default_sidecar_container(schema=False)

    @classmethod
    def _extra_processing(cls, value):
        if not value:
            return value
        V1PolyaxonSidecarContainer.from_dict(value)
        return value



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import models


class SpecModel(models.Model):
    raw_content = models.TextField(
        null=True,
        blank=True,
        help_text="The raw yaml content of the polyaxonfile/specification.",
    )
    content = models.TextField(
        null=True,
        blank=True,
        help_text="The compiled yaml content of the polyaxonfile/specification.",
    )

    class Meta:
        abstract = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from typing import Dict, Optional, Set, Union

from clipped.utils.bools import to_bool
from rest_framework import status

from django.core.handlers.asgi import ASGIRequest
from django.db import transaction
from django.http import HttpResponse
from django.urls import path

from haupt.common.endpoints.validation import validate_methods
from haupt.streams.connections.fs import AppFS
from haupt.streams.controllers.events import (
    get_archived_operation_events,
    get_archived_operation_events_and_assets,
    get_archived_operation_resources,
    get_archived_operations_events,
)
from haupt.streams.endpoints.base import UJSONResponse
from haupt.streams.endpoints.utils import redirect_file
from traceml.artifacts import V1ArtifactKind
from traceml.events import V1Events
from traceml.processors.importance_processors import calculate_importance_correlation


@transaction.non_atomic_requests
async def get_multi_run_events(
    request: ASGIRequest,
    namespace: str,
    owner: str,
    project: str,
    event_kind: str,
    methods: Optional[Dict] = None,
) -> Union[UJSONResponse, HttpResponse]:
    validate_methods(request, methods)
    force = to_bool(request.GET.get("force"), handle_none=True)
    if event_kind not in V1ArtifactKind.to_set():
        return HttpResponse(
            content="received an unrecognisable event {}.".format(event_kind),
            status=status.HTTP_400_BAD_REQUEST,
        )
    run_uuids = request.GET["runs"]
    event_names = request.GET["names"]
    orient = request.GET.get("orient")
    sample = request.GET.get("sample")
    orient = orient or V1Events.ORIENT_DICT
    event_names = {e for e in event_names.split(",") if e} if event_names else set([])
    run_uuids = {e for e in run_uuids.split(",") if e} if run_uuids else set([])
    events = await get_archived_operations_events(
        fs=await AppFS.get_fs(),
        run_uuids=run_uuids,
        event_kind=event_kind,
        event_names=event_names,
        orient=orient,
        check_cache=not force,
        sample=sample,
    )
    return UJSONResponse({"data": events})


async def get_package_event_assets(
    run_uuid: str, event_kind: str, event_names: Set[str], force: bool
) -> HttpResponse:
    archived_path = await get_archived_operation_events_and_assets(
        fs=await AppFS.get_fs(),
        run_uuid=run_uuid,
        event_kind=event_kind,
        event_names=event_names,
        check_cache=not force,
    )
    if not archived_path:
        return HttpResponse(
            content="Artifact not found: filepath={}".format(archived_path),
            status=status.HTTP_404_NOT_FOUND,
        )
    return await redirect_file(archived_path)


@transaction.non_atomic_requests
async def get_run_events(
    request: ASGIRequest,
    namespace: str,
    owner: str,
    project: str,
    run_uuid: str,
    event_kind: str,
    methods: Optional[Dict] = None,
) -> Union[UJSONResponse, HttpResponse]:
    validate_methods(request, methods)
    force = to_bool(request.GET.get("force"), handle_none=True)
    pkg_assets = to_bool(request.GET.get("pkg_assets"), handle_none=True)
    if event_kind not in V1ArtifactKind.to_set():
        return HttpResponse(
            content="received an unrecognisable event {}.".format(event_kind),
            status=status.HTTP_400_BAD_REQUEST,
        )
    event_names = request.GET["names"]
    orient = request.GET.get("orient")
    sample = request.GET.get("sample")
    orient = orient or V1Events.ORIENT_DICT
    event_names = {e for e in event_names.split(",") if e} if event_names else set([])
    if pkg_assets:
        return await get_package_event_assets(
            run_uuid=run_uuid,
            event_kind=event_kind,
            event_names=event_names,
            force=force,
        )
    events = await get_archived_operation_events(
        fs=await AppFS.get_fs(),
        run_uuid=run_uuid,
        event_kind=event_kind,
        event_names=event_names,
        orient=orient,
        check_cache=not force,
        sample=sample,
    )
    return UJSONResponse({"data": events})


@transaction.non_atomic_requests
async def get_run_resources(
    request: ASGIRequest,
    namespace: str,
    owner: str,
    project: str,
    run_uuid: str,
    methods: Optional[Dict] = None,
) -> UJSONResponse:
    validate_methods(request, methods)
    event_names = request.GET.get("names")
    orient = request.GET.get("orient")
    force = to_bool(request.GET.get("force"), handle_none=True)
    sample = request.GET.get("sample")
    orient = orient or V1Events.ORIENT_DICT
    event_names = {e for e in event_names.split(",") if e} if event_names else set([])
    events = await get_archived_operation_resources(
        fs=await AppFS.get_fs(),
        run_uuid=run_uuid,
        event_kind=V1ArtifactKind.METRIC,
        event_names=event_names,
        orient=orient,
        check_cache=not force,
        sample=sample,
    )
    return UJSONResponse({"data": events})


@transaction.non_atomic_requests
async def get_run_importance_correlation(
    request: ASGIRequest,
    namespace: str,
    owner: str,
    project: str,
    run_uuid: str,
    methods: Optional[Dict] = None,
) -> UJSONResponse:
    validate_methods(request, methods)
    body = await request.json()
    data = body.get("data")
    data = data or {}
    params = data.get("params")
    metrics = data.get("metrics")
    return UJSONResponse(
        {"data": calculate_importance_correlation(metrics=metrics, params=params)}
    )


URLS_RUNS_MULTI_EVENTS = (
    "<str:namespace>/<str:owner>/<str:project>/runs/multi/events/<str:event_kind>"
)
URLS_RUNS_EVENTS = (
    "<str:namespace>/<str:owner>/<str:project>/runs/"
    "<str:run_uuid>/events/<str:event_kind>"
)
URLS_RUNS_RESOURCES = (
    "<str:namespace>/<str:owner>/<str:project>/runs/" "<str:run_uuid>/resources"
)
URLS_RUNS_IMPORTANCE_CORRELATION = (
    "<str:namespace>/<str:owner>/<str:project>/runs/" "<str:run_uuid>/importance"
)

# fmt: off
events_routes = [
    path(
        URLS_RUNS_MULTI_EVENTS,
        get_multi_run_events,
        name="multi_run_events",
        kwargs=dict(methods=["GET"]),
    ),
    path(
        URLS_RUNS_RESOURCES,
        get_run_resources,
        name="resources",
        kwargs=dict(methods=["GET"]),
    ),
    path(
        URLS_RUNS_IMPORTANCE_CORRELATION,
        get_run_importance_correlation,
        name="importance",
        kwargs=dict(methods=["POST"]),
    ),
    path(
        URLS_RUNS_EVENTS,
        get_run_events,
        name="events",
        kwargs=dict(methods=["GET"]),
    ),
]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.apps import AppConfig


class CommandsConfig(AppConfig):
    name = "haupt.common.commands"
    verbose_name = "commands"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Optional

from haupt.proxies.generators.base import write_to_conf_file
from haupt.proxies.schemas.forward import get_forward_cmd


def generate_forward_proxy_cmd(path: Optional[str] = None):
    cmd = get_forward_cmd()
    if cmd:
        write_to_conf_file("forward_proxy", cmd, path, "sh")



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import os

from typing import Any

from haupt.common.conf.exceptions import ConfException
from haupt.common.conf.handler import BaseConfHandler
from haupt.common.options.option import Option


class EnvConfHandler(BaseConfHandler):
    def get(self, option: Option, **kwargs) -> Any:  # pylint:disable=arguments-differ
        value = os.environ.get(option.key)
        if value:
            return option.parse(value)
        if not option.is_optional:
            raise ConfException(
                "The config option `{}` was not found or not correctly "
                "set on the settings backend.".format(option.key)
            )
        return option.default_value()

    def set(  # pylint:disable=arguments-differ
        self, option: Option, value: Any, **kwargs
    ):
        os.environ[option.key] = str(value)

    def delete(self, option: Option, **kwargs):  # pylint:disable=arguments-differ
        os.environ.pop(option.key, None)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import List, Optional

from haupt.common.config_manager import ConfigManager
from polyaxon.env_vars.keys import (
    EV_KEYS_PLATFORM_HOST,
    EV_KEYS_UI_ADMIN_ENABLED,
    EV_KEYS_UI_ASSETS_VERSION,
    EV_KEYS_UI_BASE_URL,
    EV_KEYS_UI_ENABLED,
    EV_KEYS_UI_IN_SANDBOX,
    EV_KEYS_UI_OFFLINE,
)


def set_ui(context, config: ConfigManager, processors: Optional[List[str]] = None):
    context["ROOT_URLCONF"] = "{}.urls".format(config.config_module)
    platform_host = config.get(EV_KEYS_PLATFORM_HOST, "str", is_optional=True)
    context["PLATFORM_HOST"] = platform_host

    def get_allowed_hosts():
        allowed_hosts = config.get(
            "POLYAXON_ALLOWED_HOSTS",
            "str",
            is_optional=True,
            is_list=True,
            default=["*"],
        )  # type: list
        if platform_host:
            allowed_hosts.append(platform_host)
        if ".polyaxon.com" not in allowed_hosts:
            allowed_hosts.append(".polyaxon.com")
        pod_ip = config.get("POLYAXON_POD_IP", "str", is_optional=True)
        if pod_ip:
            allowed_hosts.append(pod_ip)
        host_ip = config.get("POLYAXON_HOST_IP", "str", is_optional=True)
        if host_ip:
            host_cidr = ".".join(host_ip.split(".")[:-1])
            allowed_hosts += ["{}.{}".format(host_cidr, i) for i in range(255)]

        return allowed_hosts

    context["ALLOWED_HOSTS"] = get_allowed_hosts()

    processors = processors or []
    if not config.is_streams_service:
        processors = [
            "django.contrib.auth.context_processors.auth",
            "django.contrib.messages.context_processors.messages",
        ] + processors
    processors = [
        "django.template.context_processors.debug",
        "django.template.context_processors.request",
        "django.template.context_processors.media",
        "django.template.context_processors.static",
        "django.template.context_processors.tz",
        "haupt.common.settings.context_processors.version",
        "haupt.common.settings.context_processors.ui_assets_version",
        "haupt.common.settings.context_processors.ui_base_url",
        "haupt.common.settings.context_processors.ui_offline",
        "haupt.common.settings.context_processors.ui_enabled",
        "haupt.common.settings.context_processors.ui_in_sandbox",
    ] + processors

    context["FRONTEND_DEBUG"] = config.get(
        "POLYAXON_FRONTEND_DEBUG", "bool", is_optional=True, default=False
    )

    template_debug = (
        config.get("DJANGO_TEMPLATE_DEBUG", "bool", is_optional=True)
        or config.is_debug_mode
    )
    context["UI_ADMIN_ENABLED"] = config.get(
        EV_KEYS_UI_ADMIN_ENABLED, "bool", is_optional=True, default=False
    )
    base_url = config.get(EV_KEYS_UI_BASE_URL, "str", is_optional=True)
    if base_url:
        context["UI_BASE_URL"] = base_url
        context["FORCE_SCRIPT_NAME"] = base_url
    else:
        context["UI_BASE_URL"] = "/"
    context["UI_ASSETS_VERSION"] = config.get(
        EV_KEYS_UI_ASSETS_VERSION, "str", is_optional=True, default=""
    )
    context["UI_OFFLINE"] = config.get(
        EV_KEYS_UI_OFFLINE, "bool", is_optional=True, default=False
    )
    context["UI_ENABLED"] = config.get(
        EV_KEYS_UI_ENABLED, "bool", is_optional=True, default=True
    )
    context["UI_IN_SANDBOX"] = config.get(
        EV_KEYS_UI_IN_SANDBOX, "bool", is_optional=True, default=False
    )
    context["TEMPLATES_DEBUG"] = template_debug
    context["LIST_TEMPLATE_CONTEXT_PROCESSORS"] = processors
    context["TEMPLATES"] = [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "APP_DIRS": True,
            "OPTIONS": {"debug": template_debug, "context_processors": processors},
        }
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.conf import settings

from haupt.common.auditor.manager import event_manager
from haupt.common.auditor.service import AuditorService
from haupt.common.service_interface import LazyServiceWrapper


def get_auditor_backend_path():
    return settings.AUDITOR_BACKEND or "haupt.common.auditor.service.AuditorService"


def get_auditor_options():
    return {
        "auditor_events_task": settings.AUDITOR_EVENTS_TASK,
        "workers_service": settings.WORKERS_SERVICE,
        "executor_service": settings.EXECUTOR_SERVICE or "haupt.orchestration.executor",
    }


backend = LazyServiceWrapper(
    backend_base=AuditorService,
    backend_path=get_auditor_backend_path(),
    options=get_auditor_options(),
)
backend.expose(locals())

subscribe = event_manager.subscribe



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.


from django.contrib.admin import ModelAdmin


class ReadOnlyAdmin(ModelAdmin):
    """Disables all editing capabilities."""

    actions = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # pylint:disable=protected-access
        readonly_fields = [field.name for field in self.model._meta.get_fields()]
        if self.readonly_fields:
            self.readonly_fields += tuple(readonly_fields)
        else:
            self.readonly_fields = readonly_fields

    def change_view(self, request, object_id, form_url="", extra_context=None):
        extra_context = extra_context or {}
        extra_context["show_save_and_continue"] = False
        extra_context["show_save"] = False
        return super(ReadOnlyAdmin, self).change_view(
            request, object_id, extra_context=extra_context
        )

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def save_model(self, request, obj, form, change):
        pass

    def delete_model(self, request, obj):
        pass

    def save_related(self, request, form, formsets, change):
        pass


class DiffModelAdmin(ModelAdmin):
    """Make diff model fields read-only."""

    readonly_fields = ("created_at", "updated_at")



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.options.option import Option, OptionScope, OptionStores

LOGGING = "LOGGING"
DEBUG = "DEBUG"
PROTOCOL = "PROTOCOL"
CELERY_BROKER_BACKEND = "CELERY_BROKER_BACKEND"
CELERY_BROKER_URL = "CELERY_BROKER_URL"
SECRET_INTERNAL_TOKEN = "SECRET_INTERNAL_TOKEN"  # noqa
HEALTH_CHECK_WORKER_TIMEOUT = "HEALTH_CHECK_WORKER_TIMEOUT"
SCHEDULER_ENABLED = "SCHEDULER_ENABLED"
UI_ADMIN_ENABLED = "UI_ADMIN_ENABLED"
UI_ASSETS_VERSION = "UI_ASSETS_VERSION"
UI_BASE_URL = "UI_BASE_URL"
UI_OFFLINE = "UI_OFFLINE"
UI_ENABLED = "UI_ENABLED"
UI_IN_SANDBOX = "UI_IN_SANDBOX"

OPTIONS = {
    LOGGING,
    DEBUG,
    PROTOCOL,
    CELERY_BROKER_BACKEND,
    CELERY_BROKER_URL,
    SECRET_INTERNAL_TOKEN,
    HEALTH_CHECK_WORKER_TIMEOUT,
    SCHEDULER_ENABLED,
    UI_ADMIN_ENABLED,
    UI_ASSETS_VERSION,
    UI_BASE_URL,
    UI_OFFLINE,
    UI_ENABLED,
    UI_IN_SANDBOX,
}


class Logging(Option):
    key = LOGGING
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = False
    is_list = False
    store = OptionStores.SETTINGS
    typing = "dict"
    default = None
    options = None


class Debug(Option):
    key = DEBUG
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = False
    is_list = False
    store = OptionStores.SETTINGS
    typing = "dict"
    default = None
    options = None


class Protocol(Option):
    key = PROTOCOL
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = False
    is_list = False
    store = OptionStores.SETTINGS
    typing = "str"
    default = None
    options = None


class CeleryBrokerBackend(Option):
    key = CELERY_BROKER_BACKEND
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = False
    is_list = False
    store = OptionStores.SETTINGS
    typing = "str"
    default = None
    options = None


class CeleryBrokerUrl(Option):
    key = CELERY_BROKER_URL
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = False
    is_list = False
    store = OptionStores.SETTINGS
    typing = "str"
    default = None
    options = None


class SecretInternalToken(Option):
    key = SECRET_INTERNAL_TOKEN
    scope = OptionScope.GLOBAL
    is_secret = True
    is_optional = False
    is_list = False
    store = OptionStores.SETTINGS
    typing = "str"
    default = None
    options = None


class HealthCheckWorkerTimeout(Option):
    key = HEALTH_CHECK_WORKER_TIMEOUT
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    store = OptionStores.SETTINGS
    typing = "int"
    default = 4
    options = None


class SchedulerEnabled(Option):
    key = SCHEDULER_ENABLED
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    store = OptionStores.SETTINGS
    typing = "bool"
    default = True
    options = None


class UiAdminEnabled(Option):
    key = UI_ADMIN_ENABLED
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    store = OptionStores.SETTINGS
    typing = "bool"
    default = True
    options = None


class UiAssetsVersion(Option):
    key = UI_ASSETS_VERSION
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    store = OptionStores.SETTINGS
    typing = "str"
    default = ""
    options = None


class UiBaseUrl(Option):
    key = UI_BASE_URL
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    store = OptionStores.SETTINGS
    typing = "str"
    default = "/"
    options = None


class UiOffline(Option):
    key = UI_OFFLINE
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    store = OptionStores.SETTINGS
    typing = "bool"
    default = False
    options = None


class UiEnabled(Option):
    key = UI_ENABLED
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    store = OptionStores.SETTINGS
    typing = "bool"
    default = False
    options = None


class UiInSandbox(Option):
    key = UI_IN_SANDBOX
    scope = OptionScope.GLOBAL
    is_secret = False
    is_optional = True
    is_list = False
    store = OptionStores.SETTINGS
    typing = "bool"
    default = False
    options = None



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import click

from haupt.cli.proxies import proxy
from haupt.cli.sandbox import sandbox
from haupt.cli.server import server
from haupt.cli.streams import streams
from haupt.cli.viewer import viewer
from polyaxon.logger import clean_outputs, configure_logger


@click.group()
@click.option(
    "-v", "--verbose", is_flag=True, default=False, help="Turn on debug logging"
)
@clean_outputs
def cli(verbose):
    """Haupt -  Lineage metadata API, artifacts streams, sandbox, ML-API, and spaces for Polyaxon.

    Check the help available for each command listed below by appending `-h`.
    """
    configure_logger(verbose)


cli.add_command(proxy)
cli.add_command(sandbox)
cli.add_command(server)
cli.add_command(streams)
cli.add_command(viewer)


def main():
    cli(auto_envvar_prefix="HAUPT_CLI")



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
import os

from typing import Dict, Optional

from rest_framework import status

from django.core.handlers.asgi import ASGIRequest
from django.http import HttpResponse
from django.urls import path

from haupt import settings
from haupt.common.endpoints.validation import validate_methods
from haupt.streams.endpoints.base import ConfigResponse, UJSONResponse
from polyaxon.contexts import paths as ctx_paths
from polyaxon.lifecycle import V1ProjectFeature


async def get_run_details(
    request: ASGIRequest,
    owner: str,
    project: str,
    run_uuid: str,
    methods: Optional[Dict] = None,
) -> HttpResponse:
    validate_methods(request, methods)
    subpath = os.path.join(run_uuid, ctx_paths.CONTEXT_LOCAL_RUN)
    data_path = settings.SANDBOX_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    if not os.path.exists(data_path) or not os.path.isfile(data_path):
        return HttpResponse(status=status.HTTP_404_NOT_FOUND)

    with open(data_path, "r") as config_file:
        config_str = config_file.read()
    return ConfigResponse(config_str)


async def get_run_artifact_lineage(
    request: ASGIRequest,
    owner: str,
    project: str,
    run_uuid: str,
    methods: Optional[Dict] = None,
) -> HttpResponse:
    validate_methods(request, methods)
    subpath = os.path.join(run_uuid, ctx_paths.CONTEXT_LOCAL_LINEAGES)
    data_path = settings.SANDBOX_CONFIG.get_store_path(
        subpath=subpath, entity=V1ProjectFeature.RUNTIME
    )
    if not os.path.exists(data_path) or not os.path.isfile(data_path):
        return HttpResponse(status=status.HTTP_404_NOT_FOUND)

    with open(data_path, "r") as config_file:
        config_str = config_file.read()
        config_str = f'{{"results": {config_str}}}'

    return ConfigResponse(config_str)


async def list_runs(
    request: ASGIRequest, owner: str, project: str, methods: Optional[Dict] = None
) -> HttpResponse:
    validate_methods(request, methods)
    # project = request.path_params["project"]
    data_path = settings.SANDBOX_CONFIG.get_store_path(
        subpath="", entity=V1ProjectFeature.RUNTIME
    )
    if not os.path.exists(data_path) or not os.path.isdir(data_path):
        return HttpResponse(status=status.HTTP_404_NOT_FOUND)

    data = []
    for run in os.listdir(data_path):
        run_path = os.path.join(data_path, run, ctx_paths.CONTEXT_LOCAL_RUN)
        if not os.path.exists(run_path) or not os.path.isfile(run_path):
            continue

        with open(run_path, "r") as config_file:
            data.append(config_file.read())
    data_str = ",".join(data)
    config_str = f'{{"results": [{data_str}], "count": {len(data)}}}'
    return ConfigResponse(config_str)


async def get_project_details(
    request: ASGIRequest,
    owner: str,
    project: str,
    methods: Optional[Dict] = None,
) -> HttpResponse:
    validate_methods(request, methods)
    data_path = settings.SANDBOX_CONFIG.get_store_path(
        subpath=project, entity="project"
    )
    if not os.path.exists(data_path) or not os.path.isdir(data_path):
        return HttpResponse(status=status.HTTP_404_NOT_FOUND)

    data_path = os.path.join(data_path, ctx_paths.CONTEXT_LOCAL_PROJECT)
    if os.path.exists(data_path) and os.path.isfile(data_path):
        with open(data_path, "r") as config_file:
            config_str = config_file.read()
        return ConfigResponse(config_str)

    # Use basic project configuration
    return UJSONResponse({"name": project})


async def list_projects(
    request: ASGIRequest, owner: str, methods: Optional[Dict] = None
) -> HttpResponse:
    validate_methods(request, methods)
    data_path = settings.SANDBOX_CONFIG.get_store_path(subpath="", entity="project")
    if not os.path.exists(data_path) or not os.path.isdir(data_path):
        return HttpResponse(status=status.HTTP_404_NOT_FOUND)

    data = []
    for proj in os.listdir(data_path):
        data_path = os.path.join(data_path, proj, ctx_paths.CONTEXT_LOCAL_PROJECT)
        if os.path.exists(data_path) and os.path.isfile(data_path):
            with open(data_path, "r") as config_file:
                data.append(config_file.read())
        else:
            data.append(f'{{"name": "{proj}"}}')

    data_str = ",".join(data)
    config_str = f'{{"results": [{data_str}], "count": {len(data)}}}'
    return ConfigResponse(config_str)


URLS_RUNS_DETAILS = "<str:owner>/<str:project>/runs/<str:run_uuid>/"
URLS_RUNS_STATUSES = "<str:owner>/<str:project>/runs/<str:run_uuid>/statuses"
URLS_RUNS_LINEAGE_ARTIFACTS = (
    "<str:owner>/<str:project>/runs/<str:run_uuid>/lineage/artifacts"
)
URLS_RUNS_LIST = "<str:owner>/<str:project>/runs/"
URLS_PROJECTS_LIST = "<str:owner>/projects/list"
URLS_PROJECTS_DETAILS = "<str:owner>/<str:project>/"

# fmt: off
local_sandbox_routes = [
    path(
        URLS_RUNS_DETAILS,
        get_run_details,
        name="run_details",
        kwargs=dict(methods=["GET"]),
    ),
    path(
        URLS_RUNS_STATUSES,
        get_run_details,
        name="run_details",
        kwargs=dict(methods=["GET"]),
    ),
    path(
        URLS_RUNS_LINEAGE_ARTIFACTS,
        get_run_artifact_lineage,
        name="run_artifact_lineage",
        kwargs=dict(methods=["GET"]),
    ),
    path(
        URLS_RUNS_LIST,
        list_runs,
        name="list_runs",
        kwargs=dict(methods=["GET"]),
    ),
    path(
        URLS_PROJECTS_LIST,
        list_projects,
        name="project_details",
        kwargs=dict(methods=["GET"]),
    ),
    path(
        URLS_PROJECTS_DETAILS,
        get_project_details,
        name="project_details",
        kwargs=dict(methods=["GET"]),
    ),
]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt import settings
from haupt.proxies.schemas.base import get_config

OPTIONS = """
error_log {root}/error.log {level};
"""


def get_logging_config():
    log_level = settings.PROXIES_CONFIG.log_level
    if log_level and log_level.lower() == "warning":
        log_level = "warn"
    return get_config(
        options=OPTIONS,
        indent=0,
        root=settings.PROXIES_CONFIG.logs_root,
        level=log_level,
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from django.conf import settings

from haupt.apis.endpoints.project import ProjectResourceListEndpoint
from haupt.apis.methods import project_resources as methods
from haupt.apis.serializers.artifacts import (
    RunArtifactLightSerializer,
    RunArtifactSerializer,
)
from haupt.apis.serializers.project_resources import (
    OfflineRunSerializer,
    OperationCreateSerializer,
    RunSerializer,
)
from haupt.common.apis.filters import OrderingFilter, QueryFilter
from haupt.common.apis.paginator import LargeLimitOffsetPagination
from haupt.common.endpoints.base import (
    CreateEndpoint,
    DestroyEndpoint,
    ListEndpoint,
    PostEndpoint,
)
from haupt.db.queries import artifacts as artifacts_queries
from haupt.db.queries import runs as runs_queries
from haupt.db.queries.artifacts import clean_sqlite_distinct_artifacts
from haupt.db.queries.runs import DEFAULT_COLUMNS_DEFER
from haupt.db.query_managers.artifact import ArtifactQueryManager
from haupt.db.query_managers.run import RunQueryManager


class ProjectRunsTagView(ProjectResourceListEndpoint, PostEndpoint):
    def post(self, request, *args, **kwargs):
        return methods.create_runs_tags(view=self, request=request, *args, **kwargs)


class ProjectRunsStopView(ProjectResourceListEndpoint, PostEndpoint):
    def post(self, request, *args, **kwargs):
        return methods.stop_runs(
            view=self, request=request, actor=self.project.actor, *args, **kwargs
        )


class ProjectRunsApproveView(ProjectResourceListEndpoint, PostEndpoint):
    def post(self, request, *args, **kwargs):
        return methods.approve_runs(
            view=self, request=request, actor=self.project.actor, *args, **kwargs
        )


class ProjectRunsDeleteView(ProjectResourceListEndpoint, DestroyEndpoint):
    def delete(self, request, *args, **kwargs):
        return methods.delete_runs(
            view=self, request=request, actor=self.project.actor, *args, **kwargs
        )


class ProjectRunsListView(ProjectResourceListEndpoint, ListEndpoint, CreateEndpoint):
    queryset = runs_queries.run_model.all.defer(*DEFAULT_COLUMNS_DEFER)
    filter_backends = (QueryFilter, OrderingFilter)
    query_manager = RunQueryManager
    check_alive = RunQueryManager.CHECK_ALIVE
    ordering = RunQueryManager.FIELDS_DEFAULT_ORDERING
    ordering_fields = RunQueryManager.FIELDS_ORDERING
    ordering_proxy_fields = RunQueryManager.FIELDS_ORDERING_PROXY
    serializer_class_mapping = {
        "GET": RunSerializer,
        "POST": OperationCreateSerializer,
    }

    def perform_create(self, serializer):
        serializer.save(project=self.project)


class ProjectRunsSyncView(ProjectResourceListEndpoint, CreateEndpoint):
    queryset = runs_queries.run_model.all.all()
    serializer_class = OfflineRunSerializer

    def perform_create(self, serializer):
        serializer.save(project=self.project)


class ProjectRunsArtifactsView(ProjectResourceListEndpoint, ListEndpoint):
    filter_backends = (QueryFilter, OrderingFilter)
    query_manager = ArtifactQueryManager
    check_alive = ArtifactQueryManager.CHECK_ALIVE
    ordering = ArtifactQueryManager.FIELDS_DEFAULT_ORDERING
    ordering_fields = ArtifactQueryManager.FIELDS_ORDERING
    ordering_proxy_fields = ArtifactQueryManager.FIELDS_ORDERING_PROXY
    pagination_class = LargeLimitOffsetPagination

    def enrich_queryset(self, queryset):
        return queryset.filter(run__project=self.project)

    def get_queryset(self):
        mode = self.request.query_params.get("mode")
        if not mode:
            return artifacts_queries.project_runs_artifacts
        return artifacts_queries.project_runs_artifacts_distinct

    def get_serializer_class(self):
        mode = self.request.query_params.get("mode")
        if not mode:
            return RunArtifactSerializer
        return RunArtifactLightSerializer

    def list(self, request, *args, **kwargs):
        response = super().list(request, *args, **kwargs)
        mode = self.request.query_params.get("mode")
        if settings.DB_ENGINE_NAME == "sqlite" and mode:
            (
                response.data["results"],
                response.data["count"],
            ) = clean_sqlite_distinct_artifacts(response.data["results"])
            return response
        return response



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import factory

from haupt.db.abstracts.getter import get_artifact_model
from traceml.artifacts import V1ArtifactKind


class ArtifactFactory(factory.django.DjangoModelFactory):
    name = "accuracy"
    kind = V1ArtifactKind.METRIC
    summary = {"last_value": 0.9, "max_value": 0.9, "min_value": 0.1}
    path = "accuracy"

    class Meta:
        model = get_artifact_model()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.background.celeryp.queues import CeleryCoreQueues
from haupt.background.celeryp.tasks import CoreSchedulerCeleryTasks

SCHEDULER_CORE_ROUTES = {
    # Scheduler health
    CoreSchedulerCeleryTasks.SCHEDULER_HEALTH: {
        "queue": CeleryCoreQueues.SCHEDULER_HEALTH
    },
    # compiler
    CoreSchedulerCeleryTasks.RUNS_PREPARE: {
        "queue": CeleryCoreQueues.SCHEDULER_COMPILER
    },
    CoreSchedulerCeleryTasks.RUNS_START: {"queue": CeleryCoreQueues.SCHEDULER_COMPILER},
    CoreSchedulerCeleryTasks.RUNS_BUILT: {"queue": CeleryCoreQueues.SCHEDULER_COMPILER},
    # Scheduler runs
    CoreSchedulerCeleryTasks.RUNS_STOP: {"queue": CeleryCoreQueues.SCHEDULER_RUNS},
    CoreSchedulerCeleryTasks.RUNS_DELETE: {"queue": CeleryCoreQueues.SCHEDULER_RUNS},
    # Scheduler artifacts
    CoreSchedulerCeleryTasks.RUNS_SET_ARTIFACTS: {
        "queue": CeleryCoreQueues.SCHEDULER_ARTIFACTS,
        "priority": 0,
    },
}


def get_routes():
    routes = {}
    routes.update(SCHEDULER_CORE_ROUTES)
    return routes



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import models

from haupt.db.abstracts.diff import DiffModel
from haupt.db.abstracts.getter import get_db_model_name
from haupt.db.abstracts.state import StateModel
from traceml.artifacts import V1ArtifactKind


class BaseArtifact(DiffModel, StateModel):
    name = models.CharField(max_length=256, db_index=True)
    kind = models.CharField(
        max_length=12,
        db_index=True,
        choices=V1ArtifactKind.to_choices(),
    )
    path = models.CharField(max_length=256, blank=True, null=True)
    summary = models.JSONField()

    class Meta:
        abstract = True


class BaseArtifactLineage(DiffModel):
    run = models.ForeignKey(
        get_db_model_name("Run"),
        on_delete=models.CASCADE,
        related_name="artifacts_lineage",
    )
    artifact = models.ForeignKey(
        get_db_model_name("Artifact"),
        on_delete=models.CASCADE,
        related_name="runs_lineage",
    )
    is_input = models.BooleanField(null=True, blank=True, default=False)

    class Meta:
        abstract = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.core.serializers.json import DjangoJSONEncoder
from django.db import models

from polyaxon.lifecycle import V1Statuses


class StatusModel(models.Model):
    status = models.CharField(
        max_length=16,
        blank=True,
        null=True,
        db_index=True,
        default=V1Statuses.CREATED,
        choices=V1Statuses.to_choices(),
    )
    status_conditions = models.JSONField(
        encoder=DjangoJSONEncoder, blank=True, null=True, default=dict
    )

    class Meta:
        abstract = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.query.service import QueryService
from haupt.common.service_interface import LazyServiceWrapper

backend = LazyServiceWrapper(
    backend_base=QueryService,
    backend_path="haupt.common.query.service.QueryService",
    options={},
)
backend.expose(locals())



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from clipped.utils.imports import import_string

from haupt.common.events.event_service import EventService
from haupt.common.events.registry import run
from haupt.orchestration.executor.handlers import run as run_handlers
from haupt.orchestration.executor.manager import event_manager


class ExecutorService(EventService):
    HANDLER_MAPPING = {
        run.RUN_CREATED: run_handlers.handle_run_created,
        run.RUN_RESUMED_ACTOR: run_handlers.handle_run_created,
        run.RUN_APPROVED_ACTOR: run_handlers.handle_run_approved_triggered,
        run.RUN_STOPPED_ACTOR: run_handlers.handle_run_stopped_triggered,
        run.RUN_NEW_ARTIFACTS: run_handlers.handle_new_artifacts,
        run.RUN_DELETED_ACTOR: run_handlers.handle_run_deleted,
    }

    event_manager = event_manager

    def __init__(self, workers_service=None):
        self.workers_service = workers_service
        self.workers = None

    def record_event(self, event: "Event") -> None:  # noqa: F821
        if self.workers and event.event_type in self.HANDLER_MAPPING:
            self.HANDLER_MAPPING[event.event_type](
                workers_backend=self.workers, event=event
            )

    def setup(self) -> None:
        super().setup()
        # Load default event types
        import haupt.orchestration.executor.subscriptions  # noqa

        if self.workers_service:
            self.workers = import_string(self.workers_service)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from datetime import datetime
from typing import List

from clipped.utils.json import orjson_loads
from clipped.utils.paths import delete_path

from django.core.exceptions import BadRequest

from asgiref.sync import sync_to_async
from polyaxon import settings
from polyaxon.fs.async_manager import (
    delete_file_or_dir,
    download_dir,
    open_file,
    upload_data,
)
from polyaxon.fs.types import FSSystem
from traceml.logging import V1Log, V1Logs


async def clean_tmp_logs(fs: FSSystem, run_uuid: str):
    if not settings.AGENT_CONFIG.artifacts_store:
        raise BadRequest("Run's logs was not collected, resource was not found.")
    subpath = "{}/.tmpplxlogs".format(run_uuid)
    delete_path(subpath)
    await delete_file_or_dir(fs=fs, subpath=subpath, is_file=False)


async def upload_logs(fs: FSSystem, run_uuid: str, logs: List[V1Log]):
    if not settings.AGENT_CONFIG.artifacts_store:
        raise BadRequest("Run's logs was not collected, resource was not found.")
    for c_logs in V1Logs.chunk_logs(logs):
        last_file = datetime.timestamp(c_logs.logs[-1].timestamp)
        if settings.AGENT_CONFIG.compressed_logs:
            subpath = "{}/plxlogs/{}.plx".format(run_uuid, last_file)
            await upload_data(
                fs=fs,
                subpath=subpath,
                data="{}\n{}".format(c_logs.get_csv_header(), c_logs.to_csv()),
            )
        else:
            subpath = "{}/plxlogs/{}".format(run_uuid, last_file)
            await upload_data(fs=fs, subpath=subpath, data=c_logs.to_json())


async def content_to_logs(content, logs_path):
    if not content:
        return []

    @sync_to_async
    def convert():
        # Version handling
        if ".plx" in logs_path:
            return V1Logs.read_csv(content).logs
        # Chunked logs
        return orjson_loads(content).get("logs", [])

    return await convert()


async def download_logs_file(
    fs: FSSystem, run_uuid: str, last_file: str, check_cache: bool = True
) -> (str, str):
    subpath = "{}/plxlogs/{}".format(run_uuid, last_file)
    content = await open_file(fs=fs, subpath=subpath, check_cache=check_cache)

    return await content_to_logs(content, subpath)


async def download_tmp_logs(fs: FSSystem, run_uuid: str) -> str:
    subpath = "{}/.tmpplxlogs".format(run_uuid)
    delete_path(subpath)
    return await download_dir(fs=fs, subpath=subpath)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import os

import click


@click.command()
@click.option(
    "--host",
    help="The service host.",
)
@click.option(
    "--port",
    type=int,
    help="The service port.",
)
@click.option(
    "--workers",
    type=int,
    help="Number of workers.",
)
@click.option(
    "--per-core",
    is_flag=True,
    default=False,
    help="To enable workers per core.",
)
@click.option(
    "--path",
    help="The service host.",
)
@click.option(
    "--uds",
    help="UNIX domain socket binding.",
)
def sandbox(
    host: str,
    port: int,
    workers: int,
    per_core: bool,
    path: str,
    uds: str,
):
    """Start a new sandbox session."""
    from haupt.cli.runners.sandbox import start
    from polyaxon.env_vars.keys import EV_KEYS_SANDBOX_ROOT

    if path:
        os.environ[EV_KEYS_SANDBOX_ROOT] = path

    return start(host=host, port=port, workers=workers, per_core=per_core, uds=uds)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import factory

from haupt.db.abstracts.getter import get_run_model
from haupt.db.factories.projects import ProjectFactory
from haupt.db.factories.users import UserFactory


class RunFactory(factory.django.DjangoModelFactory):
    user = factory.SubFactory(UserFactory)
    project = factory.SubFactory(ProjectFactory)
    original = None
    pipeline = None
    is_managed = False

    class Meta:
        model = get_run_model()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.core.management import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        from haupt.common import workers
        from kombu import Exchange

        Exchange(
            "internal", type="topic", channel=workers.app.connection().channel()
        ).declare()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.db import models


class DurationModel(models.Model):
    started_at = models.DateTimeField(blank=True, null=True)
    finished_at = models.DateTimeField(blank=True, null=True)
    wait_time = models.IntegerField(blank=True, null=True)
    duration = models.IntegerField(blank=True, null=True)

    class Meta:
        abstract = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import logging

from typing import Dict, List

from haupt.background.celeryp.tasks import CoreSchedulerCeleryTasks
from haupt.common import workers
from haupt.orchestration.scheduler import manager
from haupt.polyconf.settings import Intervals

_logger = logging.getLogger("polyaxon.scheduler")


@workers.app.task(name=CoreSchedulerCeleryTasks.RUNS_PREPARE, ignore_result=True)
def runs_prepare(run_id):
    if manager.runs_prepare(run_id=run_id, run=None):
        workers.send(CoreSchedulerCeleryTasks.RUNS_START, kwargs={"run_id": run_id})


@workers.app.task(name=CoreSchedulerCeleryTasks.RUNS_START, ignore_result=True)
def runs_start(run_id):
    manager.runs_start(run_id=run_id, run=None)


@workers.app.task(name=CoreSchedulerCeleryTasks.RUNS_BUILT, ignore_result=True)
def runs_built(run_id):
    # Move to CE
    return run_id


@workers.app.task(name=CoreSchedulerCeleryTasks.RUNS_SET_ARTIFACTS, ignore_result=True)
def runs_set_artifacts(run_id, artifacts: List[Dict]):
    manager.runs_set_artifacts(run_id=run_id, run=None, artifacts=artifacts)


@workers.app.task(
    name=CoreSchedulerCeleryTasks.RUNS_STOP,
    bind=True,
    max_retries=3,
    ignore_result=True,
)
def runs_stop(self, run_id, update_status=False, message=None):
    stopped = manager.runs_stop(
        run_id=run_id, run=None, update_status=update_status, message=message
    )
    if not stopped and self.request.retries < 2:
        _logger.info("Trying again to delete job `%s` in run.", run_id)
        self.retry(countdown=Intervals.RUNS_SCHEDULER)
        return


@workers.app.task(name=CoreSchedulerCeleryTasks.RUNS_DELETE, ignore_result=True)
def runs_delete(run_id):
    manager.runs_delete(run_id=run_id, run=None)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt import settings
from haupt.proxies.schemas.base import get_config
from haupt.proxies.schemas.urls import get_ssl_server_name, has_https

STATIC_PROXY_OPTIONS = """
location /static/ {{
    {ssl_server_name}
    proxy_pass {service};
    proxy_pass_request_body off;
    proxy_set_header Content-Length "";
    proxy_set_header X-Real-IP $remote_addr;
    proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
    proxy_set_header X-Forwarded-Proto $scheme;
    proxy_set_header X-Origin-URI $request_uri;
    proxy_set_header X-Origin-Method $request_method;
}}
"""


def get_static_proxy_config():
    return get_config(
        options=STATIC_PROXY_OPTIONS,
        indent=0,
        service=settings.PROXIES_CONFIG.static_url.rstrip("/") + "/",
        ssl_server_name=get_ssl_server_name(settings.PROXIES_CONFIG.static_url),
    )


STATIC_LOCATION_OPTIONS = """
location /static/ {{
    alias {static_root};
    autoindex on;
    expires                   30d;
    add_header                Cache-Control private;
    gzip_static on;
}}
"""


def get_static_location_config():
    return get_config(
        options=STATIC_LOCATION_OPTIONS,
        indent=0,
        static_root=settings.PROXIES_CONFIG.static_root.rstrip("/") + "/",
    )


TMP_LOCATION_OPTIONS = """
location /tmp/ {{
    alias                     /tmp/;
    expires                   0;
    add_header                Cache-Control private;
    internal;
}}
"""


def get_tmp_location_config():
    return get_config(options=TMP_LOCATION_OPTIONS, indent=0)


ARCHIVES_LOCATION_OPTIONS = """
location {archives_root} {{
    alias                     {archives_root};
    expires                   0;
    add_header                Cache-Control private;
    set                       $x_content_length $upstream_http_x_content_length;
    add_header                X-Content-Length $x_content_length;
    internal;
}}
"""


def get_archives_root_location_config():
    return get_config(
        options=ARCHIVES_LOCATION_OPTIONS,
        indent=0,
        archives_root=settings.PROXIES_CONFIG.archives_root.rstrip("/") + "/",
    )


def get_api_locations_config():
    if settings.PROXIES_CONFIG.static_url and has_https(
        settings.PROXIES_CONFIG.static_url
    ):
        static_location = get_static_proxy_config()
    else:
        static_location = get_static_location_config()
    config = [static_location, get_tmp_location_config()]
    return "\n".join(config)


def get_streams_locations_config():
    config = [
        get_tmp_location_config(),
        get_archives_root_location_config(),
    ]
    return "\n".join(config)


def get_platform_locations_config():
    if settings.PROXIES_CONFIG.static_url and has_https(
        settings.PROXIES_CONFIG.static_url
    ):
        static_location = get_static_proxy_config()
    else:
        static_location = get_static_location_config()
    config = [
        static_location,
        get_tmp_location_config(),
        get_archives_root_location_config(),
    ]
    return "\n".join(config)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import os

from haupt.schemas.sandbox_config import SandboxConfig
from polyaxon.config.manager import ConfigManager
from polyaxon.managers.base import BaseConfigManager, ManagerVisibility


class SandboxConfigManager(BaseConfigManager):
    """Manages sandbox configuration .sandbox file."""

    VISIBILITY = ManagerVisibility.ALL
    CONFIG_FILE_NAME = ".sandbox"
    CONFIG = SandboxConfig

    @classmethod
    def get_config_from_env(cls):
        config_paths = [os.environ, {"dummy": "dummy"}]

        config = ConfigManager.read_configs(config_paths)
        return cls.CONFIG.from_dict(config.data)

    @classmethod
    def get_config_or_default(cls) -> SandboxConfig:
        if not cls.is_initialized():
            return cls.get_config_from_env()

        return cls.get_config()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import os

from pathlib import Path
from typing import List

from haupt import pkg
from polyaxon.config.manager import ConfigManager as BaseConfigManager
from polyaxon.env_vars.keys import (
    EV_KEYS_DEBUG,
    EV_KEYS_ENVIRONMENT,
    EV_KEYS_LOG_LEVEL,
    EV_KEYS_PLATFORM_CONFIG,
    EV_KEYS_SERVICE,
    EV_KEYS_TIME_ZONE,
)
from polyaxon.k8s.namespace import DEFAULT_NAMESPACE


class ConfigManager(BaseConfigManager):
    def __init__(self, **params):
        super().__init__(**params)
        self._env = self.get(
            EV_KEYS_ENVIRONMENT, "str", is_optional=True, default="local"
        )
        self._config_module = self.get(
            "POLYAXON_CONFIG_MODULE", "str", is_optional=True, default="polyconf"
        )
        self._root_dir = self.get_value("POLYAXON_CONFIG_ROOT_DIR")
        self._service = self.get(
            EV_KEYS_SERVICE, "str", is_local=True, is_optional=True
        )
        self._is_debug_mode = self.get(
            EV_KEYS_DEBUG, "bool", is_optional=True, default=False
        )
        self._db_engine_name = self.get(
            "POLYAXON_DB_ENGINE", "str", default="sqlite", is_optional=True
        )
        self._namespace = self.get(
            "POLYAXON_K8S_NAMESPACE",
            "str",
            is_optional=True,
            default=DEFAULT_NAMESPACE,
        )
        self._log_level = self.get(
            EV_KEYS_LOG_LEVEL,
            "str",
            is_local=True,
            is_optional=True,
            default="WARNING",
        ).upper()
        self._timezone = self.get(
            EV_KEYS_TIME_ZONE, "str", is_optional=True, default="UTC"
        )
        self._scheduler_enabled = self.get(
            "POLYAXON_SCHEDULER_ENABLED", "bool", is_optional=True, default=False
        )
        self._chart_version = self.get(
            "POLYAXON_CHART_VERSION", "str", is_optional=True, default=pkg.VERSION
        )
        self._redis_protocol = self.get(
            "POLYAXON_REDIS_PROTOCOL", "str", is_optional=True, default="redis"
        )
        self._broker_backend = self.get(
            "POLYAXON_BROKER_BACKEND",
            "str",
            is_optional=True,
            options=["redis", "rabbitmq"],
        )
        self._redis_password = self.get(
            "POLYAXON_REDIS_PASSWORD",
            "str",
            is_optional=True,
            is_secret=True,
            is_local=True,
        )

    @property
    def namespace(self) -> str:
        return self._namespace

    @property
    def config_module(self) -> str:
        return self._config_module

    @property
    def root_dir(self) -> Path:
        return self._root_dir

    @property
    def db_engine_name(self) -> str:
        return self._db_engine_name

    @property
    def is_sqlite_db_engine(self) -> bool:
        return self._db_engine_name == "sqlite"

    @property
    def is_pgsql_db_engine(self) -> bool:
        return self._db_engine_name == "pgsql"

    @property
    def chart_version(self) -> str:
        return self._chart_version

    @property
    def service(self) -> str:
        return self._service

    @property
    def is_streams_service(self) -> bool:
        return self.service == "streams"

    @property
    def is_monolith_service(self) -> bool:
        return self.service == "monolith"

    @property
    def is_api_service(self) -> bool:
        return self.service == "api" or self.is_monolith_service

    @property
    def is_scheduler_service(self) -> bool:
        return self.service == "scheduler" or self.is_monolith_service

    @property
    def is_debug_mode(self) -> bool:
        return self._is_debug_mode

    @property
    def env(self) -> str:
        return self._env

    @property
    def is_test_env(self) -> bool:
        return self.env == "test"

    @property
    def is_local_env(self) -> bool:
        if self.env == "local":
            return True
        return False

    @property
    def is_staging_env(self) -> bool:
        if self.env == "staging":
            return True
        return False

    @property
    def is_production_env(self) -> bool:
        if self.env == "production":
            return True
        return False

    @property
    def log_handlers(self) -> List[str]:
        return ["console"]

    @property
    def log_level(self) -> str:
        if self.is_staging_env or self.is_local_env or self.is_test_env:
            return self._log_level
        if self._log_level == "DEBUG":
            return "INFO"
        return self._log_level

    @property
    def timezone(self) -> str:
        return self._timezone

    @property
    def scheduler_enabled(self) -> bool:
        return self._scheduler_enabled

    @property
    def broker_backend(self) -> str:
        return self._broker_backend

    @property
    def is_redis_broker(self):
        return self.broker_backend == "redis"

    @property
    def is_rabbitmq_broker(self):
        return self.broker_backend == "rabbitmq"

    def get_redis_url(self, env_url_name) -> str:
        redis_url = self.get(env_url_name, "str")
        if self._redis_password:
            redis_url = ":{}@{}".format(self._redis_password, redis_url)
        return "{}://{}".format(self._redis_protocol, redis_url)

    def _get_rabbitmq_broker_url(self) -> str:
        amqp_url = self.get("POLYAXON_AMQP_URL", "str")
        rabbitmq_user = self.get("POLYAXON_RABBITMQ_USER", "str", is_optional=True)
        rabbitmq_password = self.get(
            "POLYAXON_RABBITMQ_PASSWORD",
            "str",
            is_secret=True,
            is_local=True,
            is_optional=True,
        )
        if rabbitmq_user and rabbitmq_password:
            return "amqp://{user}:{password}@{url}".format(
                user=rabbitmq_user, password=rabbitmq_password, url=amqp_url
            )

        return "amqp://{url}".format(url=amqp_url)

    def get_broker_url(self) -> str:
        if self.is_redis_broker:
            return self.get_redis_url("POLYAXON_REDIS_CELERY_BROKER_URL")
        if self.is_rabbitmq_broker:
            return self._get_rabbitmq_broker_url()


def get_config(file_path):
    def base_directory():
        root = Path(os.path.abspath(file_path))
        root.resolve()
        return root.parent.parent

    root_dir = base_directory()

    config_module = "polyconf"
    config_prefix = os.environ.get("CONFIG_PREFIX")
    if config_prefix:
        config_module = "{}.{}".format(config_prefix, config_module)
    config_values = [
        os.environ,
        {"POLYAXON_CONFIG_MODULE": config_module, "POLYAXON_CONFIG_ROOT_DIR": root_dir},
    ]

    platform_config = os.getenv(EV_KEYS_PLATFORM_CONFIG)
    if platform_config and os.path.isfile(platform_config):
        config_values.append(platform_config)

    return ConfigManager.read_configs(config_values)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from clipped.decorators.signals import ignore_raw, ignore_updates
from rest_framework.exceptions import ValidationError

from django.db.models.signals import post_save
from django.dispatch import receiver

from haupt.common import auditor
from haupt.common.events.registry.run import RUN_CREATED
from haupt.db.abstracts.getter import get_run_model


@receiver(post_save, sender=get_run_model(), dispatch_uid="run_created")
@ignore_updates
@ignore_raw
def run_created(sender, **kwargs):
    instance = kwargs["instance"]
    if instance.is_managed:
        if (instance.is_clone and instance.content is None) or (
            not instance.is_clone and instance.raw_content is None
        ):
            raise ValidationError("A managed run should have a valid specification.")
    auditor.record(event_type=RUN_CREATED, instance=instance)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import django.db.models.deletion

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("db", "0002_auto_20200807_1247"),
    ]

    operations = [
        migrations.AddField(
            model_name="run",
            name="pipeline",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="pipeline_runs",
                to="db.Run",
            ),
        ),
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.config_manager import ConfigManager
from haupt.common.settings.apps import set_apps
from haupt.common.settings.assets import set_assets
from haupt.common.settings.core import set_core
from haupt.common.settings.cors import set_cors
from haupt.common.settings.middlewares import set_base_middlewares
from haupt.common.settings.ui import set_ui
from polyaxon.env_vars.keys import EV_KEYS_UI_IN_SANDBOX


def set_streams_apps(context, config: ConfigManager):
    set_apps(
        context=context,
        config=config,
        third_party_apps=("rest_framework", "corsheaders"),
        project_apps=(
            "haupt.common.apis.apps.CommonApisConfig",
            "haupt.streams.apps.StreamsConfig",
        ),
        use_db_apps=False,
        use_staticfiles_app=context["UI_IN_SANDBOX"],
    )


def set_service(context, config: ConfigManager):
    # This is repeated because it's required for using the staticfiles app
    context["UI_IN_SANDBOX"] = config.get(
        EV_KEYS_UI_IN_SANDBOX, "bool", is_optional=True, default=False
    )
    set_streams_apps(context, config)
    set_core(context=context, config=config, use_db=False)
    set_cors(context=context, config=config)
    set_ui(context=context, config=config)
    set_base_middlewares(context=context, config=config)
    set_assets(context=context, config=config)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework import serializers
from rest_framework.exceptions import ValidationError


class IsManagedMixin(serializers.Serializer):
    is_managed = serializers.BooleanField(initial=True, default=True, allow_null=True)

    def _get_is_managed(self, value):
        return value if isinstance(value, bool) else True

    def check_if_entity_is_managed(self, attrs, entity_name, config_field="content"):
        cond = (
            "is_managed" in attrs
            and self._get_is_managed(attrs.get("is_managed"))
            and not attrs.get(config_field)
        )
        if cond:
            raise ValidationError(
                "{} expects a `{}`.".format(entity_name, config_field)
            )

    def validate_is_managed(self, value):
        return self._get_is_managed(value)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import os

import aiofiles
import nbformat

from asgiref.sync import sync_to_async
from nbconvert import HTMLExporter


async def render_notebook(archived_path: str, check_cache=True):
    html_path = archived_path.split(".ipynb")[0] + ".html"
    if os.path.exists(html_path):
        if check_cache:
            # file already exists
            return html_path
        else:
            os.remove(html_path)

    async with aiofiles.open(os.path.abspath(archived_path), mode="r") as f:
        read_data = await f.read()
        notebook = await sync_to_async(nbformat.reads)(read_data, as_version=4)
        html_exporter = HTMLExporter()
        (body, resources) = await sync_to_async(html_exporter.from_notebook_node)(
            notebook
        )
        html_file = "<style>" + resources["inlining"]["css"][0] + "</style>" + body
        async with aiofiles.open(html_path, "w") as destination:
            await destination.write(html_file)
        return html_path



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from haupt.common.apis.regex import INSTALLATION_PATTERN, NAME_PATTERN, VERSION_PATTERN

URLS_VERSIONS_INSTALLED = r"^installation/?$"
URLS_VERSIONS_LOG_HANDLER = r"^log_handler/?$"
URLS_VERSIONS_COMPATIBILITY = r"^compatibility/{}/{}/{}/?$".format(
    INSTALLATION_PATTERN, VERSION_PATTERN, NAME_PATTERN
)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from typing import Dict, List, Optional

from haupt.db.abstracts.getter import get_run_model
from haupt.db.abstracts.runs import BaseRun
from polyaxon.lifecycle import V1StatusCondition, V1Statuses
from polyaxon.polyflow import V1CompiledOperation, V1RunKind
from polyaxon.schemas import V1RunPending


def create_run(
    project_id: int,
    user_id: int,
    name: Optional[str] = None,
    description: Optional[str] = None,
    readme: Optional[str] = None,
    tags: List[int] = None,
    raw_content: Optional[str] = None,
    meta_info: Optional[Dict] = None,
) -> BaseRun:
    instance = get_run_model().objects.create(
        project_id=project_id,
        user_id=user_id,
        name=name,
        description=description,
        readme=readme,
        tags=tags,
        kind=V1RunKind.JOB,
        is_managed=False,
        raw_content=raw_content,
        meta_info=meta_info,
        status_conditions=[
            V1StatusCondition.get_condition(
                type=V1Statuses.CREATED,
                status="True",
                reason="ModelManager",
                message="Run is created",
            ).to_dict()
        ],
    )
    return instance


def base_approve_run(run: BaseRun):
    pending = run.pending
    if pending:
        new_pending = None
        if (
            (pending == V1RunPending.BUILD and run.status == V1Statuses.CREATED)
            or pending == V1RunPending.UPLOAD
        ) and run.content:
            compiled_operation = V1CompiledOperation.read(
                run.content
            )  # TODO: Use construct
            if compiled_operation.is_approved is False:
                new_pending = V1RunPending.APPROVAL
        run.pending = new_pending
        run.save(update_fields=["pending", "updated_at"])



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import os
import sys

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "haupt.polyconf.settings")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from typing import Any

import redis


class BaseRedisDb:
    REDIS_POOL = None

    @classmethod
    def _get_redis(cls) -> Any:
        return redis.Redis(
            connection_pool=cls.REDIS_POOL, retry_on_timeout=True, socket_keepalive=True
        )

    @classmethod
    def connection(cls) -> Any:
        return cls._get_redis()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.conf.handlers.env_handler import EnvConfHandler
from haupt.common.conf.handlers.settings_handler import SettingsConfHandler
from haupt.common.conf.option_service import OptionService
from haupt.common.options.option import OptionStores


class ConfService(OptionService):
    def setup(self):
        super().setup()
        self.stores[OptionStores.SETTINGS] = SettingsConfHandler()
        self.stores[OptionStores.ENV] = EnvConfHandler()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

# Setting values to None means using defaults

from haupt import pkg
from polyaxon import dist

ENCRYPTION_BACKEND = None
CONF_CHECK_OWNERSHIP = False
AUDITOR_BACKEND = None
AUDITOR_EVENTS_TASK = None
WORKERS_BACKEND = None
EXECUTOR_BACKEND = "haupt.orchestration.executor.service.ExecutorService"
WORKERS_SERVICE = "haupt.common.workers"
EXECUTOR_SERVICE = "haupt.orchestration.executor"
OPERATIONS_BACKEND = None
PLATFORM_VERSION = pkg.VERSION
PLATFORM_DIST = dist.CE
CONF_BACKEND = "haupt.common.conf.service.ConfService"
STORE_OPTION = "env"
K8S_IN_CLUSTER = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from django.conf import settings
from django.core.validators import validate_slug
from django.db import models

from haupt.db.abstracts.describable import DescribableModel
from haupt.db.abstracts.diff import DiffModel
from haupt.db.abstracts.duration import DurationModel
from haupt.db.abstracts.getter import get_db_model_name
from haupt.db.abstracts.live_state import LiveStateModel
from haupt.db.abstracts.nameable import NameableModel
from haupt.db.abstracts.readme import ReadmeModel
from haupt.db.abstracts.spec import SpecModel
from haupt.db.abstracts.status import StatusModel
from haupt.db.abstracts.tag import TagModel
from haupt.db.abstracts.uid import UuidModel
from polyaxon.lifecycle import V1Statuses
from polyaxon.polyflow import V1CloningKind, V1RunKind
from polyaxon.schemas import V1RunPending


class BaseRun(
    UuidModel,
    DiffModel,
    DurationModel,
    SpecModel,
    NameableModel,
    DescribableModel,
    ReadmeModel,
    StatusModel,
    TagModel,
    LiveStateModel,
):
    name = models.CharField(
        max_length=128,
        blank=True,
        null=True,
        default=None,
        validators=[validate_slug],
    )
    kind = models.CharField(
        max_length=12,
        db_index=True,
        choices=V1RunKind.to_choices(),
    )
    runtime = models.CharField(max_length=12, db_index=True, null=True, blank=True)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name="+",
        null=True,
        blank=True,
    )
    project = models.ForeignKey(
        get_db_model_name("Project"), on_delete=models.CASCADE, related_name="runs"
    )
    is_managed = models.BooleanField(
        default=True, help_text="If this entity is managed by the platform."
    )
    pending = models.CharField(
        max_length=8,
        null=True,
        blank=True,
        db_index=True,
        choices=V1RunPending.to_choices(),
        help_text="If this entity requires approval before it should run.",
    )
    meta_info = models.JSONField(null=True, blank=True, default=dict)
    params = models.JSONField(null=True, blank=True)
    inputs = models.JSONField(null=True, blank=True)
    outputs = models.JSONField(null=True, blank=True)
    original = models.ForeignKey(
        "self",
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        related_name="clones",
    )
    pipeline = models.ForeignKey(
        "self",
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        related_name="pipeline_runs",
    )
    cloning_kind = models.CharField(
        max_length=12,
        blank=True,
        null=True,
        choices=V1CloningKind.to_choices(),
    )
    artifacts = models.ManyToManyField(
        get_db_model_name("Artifact"),
        blank=True,
        through=get_db_model_name("ArtifactLineage"),
        related_name="runs",
    )

    class Meta:
        abstract = True
        indexes = [models.Index(fields=["name"])]

    @property
    def subpath(self):
        return self.uuid.hex

    @property
    def is_clone(self) -> bool:
        return self.original_id is not None

    @property
    def is_restart(self) -> bool:
        return self.is_clone and self.cloning_kind == V1CloningKind.RESTART

    @property
    def is_resume(self) -> bool:
        if not self.status_conditions:
            return False
        return bool(
            [c for c in self.status_conditions if c.get("type") == V1Statuses.RESUMING]
        )

    @property
    def is_copy(self) -> bool:
        return self.is_clone and self.cloning_kind == V1CloningKind.COPY

    @property
    def is_job(self):
        return self.kind == V1RunKind.JOB

    @property
    def is_service(self):
        return self.kind == V1RunKind.SERVICE

    @property
    def is_dag(self):
        return self.kind == V1RunKind.DAG

    @property
    def is_matrix(self):
        return self.kind == V1RunKind.MATRIX

    @property
    def is_schedule(self):
        return self.kind == V1RunKind.SCHEDULE

    @property
    def has_pipeline(self):
        return self.is_dag or self.is_matrix or self.is_schedule

    @property
    def has_tuner_runtime(self):
        return self.runtime == V1RunKind.TUNER

    @property
    def has_notifier_runtime(self):
        return self.runtime == V1RunKind.NOTIFIER



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.conf import settings
from django.db.models import Q

from polyaxon.pql.manager import PQLManager


class BaseQueryManager(PQLManager):
    QUERY_BACKEND = Q
    TIMEZONE = settings.TIME_ZONE



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework.exceptions import ValidationError


class NamesMixin:
    def validated_name(self, validated_data, project, query):
        name = validated_data.get("name")
        if name and query.filter(project=project, name=name).exists():
            count = query.exclude(name=name).filter(name__startswith=name).count() + 1
            validated_data["name"] = "{}-{}".format(name, count)
        return validated_data


class CatalogNamesMixin:
    def validated_name(self, validated_data, owner, query):
        name = validated_data.get("name")
        if name and query.filter(owner=owner, name=name).exists():
            raise ValidationError("An instance already exists with this name.")
        return validated_data



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Optional

from clipped.utils.http import clean_host
from pydantic import Field, StrictInt, StrictStr

from polyaxon.env_vars.keys import (
    EV_KEYS_K8S_NAMESPACE,
    EV_KEYS_SANDBOX_DEBUG,
    EV_KEYS_SANDBOX_HOST,
    EV_KEYS_SANDBOX_IS_LOCAL,
    EV_KEYS_SANDBOX_PER_CORE,
    EV_KEYS_SANDBOX_PORT,
    EV_KEYS_SANDBOX_SSL_ENABLED,
    EV_KEYS_SANDBOX_WORKERS,
)
from polyaxon.schemas.cli.agent_config import BaseAgentConfig


class SandboxConfig(BaseAgentConfig):
    _IDENTIFIER = "sandbox"

    namespace: Optional[StrictStr] = Field(
        alias=EV_KEYS_K8S_NAMESPACE, default="sandbox"
    )
    port: Optional[StrictInt] = Field(alias=EV_KEYS_SANDBOX_PORT)
    host: Optional[StrictStr] = Field(alias=EV_KEYS_SANDBOX_HOST)
    ssl_enabled: Optional[bool] = Field(alias=EV_KEYS_SANDBOX_SSL_ENABLED)
    debug: Optional[bool] = Field(alias=EV_KEYS_SANDBOX_DEBUG)
    workers: Optional[StrictInt] = Field(alias=EV_KEYS_SANDBOX_WORKERS)
    per_core: Optional[bool] = Field(alias=EV_KEYS_SANDBOX_PER_CORE)
    is_local: Optional[bool] = Field(alias=EV_KEYS_SANDBOX_IS_LOCAL)

    def __init__(
        self,
        host: Optional[str] = None,
        **data,
    ):
        super().__init__(
            host=clean_host(host) if host else host,
            **data,
        )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import random

from django.db import models

from haupt.db.managers.deleted import ArchivedManager, LiveManager, RestorableManager
from polyaxon import live_state


class LiveStateModel(models.Model):
    live_state = models.IntegerField(
        null=True,
        blank=True,
        default=live_state.STATE_LIVE,
        choices=live_state.CHOICES,
        db_index=True,
    )

    objects = LiveManager()
    restorable = RestorableManager()
    all = models.Manager()
    archived = ArchivedManager()

    class Meta:
        abstract = True

    def delete_in_progress(self, update_name=False, commit=True) -> bool:
        if self.live_state == live_state.STATE_DELETION_PROGRESSING:
            return False

        if update_name:
            self.name = "del_{}_{}".format(
                self.name, getattr(self, "uuid", random.randint(-90, 100))
            )
        self.live_state = live_state.STATE_DELETION_PROGRESSING
        if commit:
            self.save(update_fields=["name", "live_state"])
        return True

    def archive(self, commit=True) -> bool:
        if (
            self.live_state == live_state.STATE_ARCHIVED
            or self.live_state == live_state.STATE_DELETION_PROGRESSING
        ):
            return False

        self.live_state = live_state.STATE_ARCHIVED
        if commit:
            self.save(update_fields=["live_state"])
        return True

    def restore(self) -> bool:
        if self.live_state != live_state.STATE_ARCHIVED:
            return False

        self.live_state = live_state.STATE_LIVE
        self.save(update_fields=["live_state"])
        return True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Dict, Optional

from django.core.handlers.asgi import ASGIRequest
from django.db import transaction
from django.http import HttpResponse
from django.urls import path

from haupt.common.endpoints.validation import validate_methods
from haupt.streams.controllers.k8s_crd import get_k8s_operation
from haupt.streams.controllers.k8s_pods import get_pods
from haupt.streams.endpoints.base import UJSONResponse
from polyaxon import settings
from polyaxon.k8s.async_manager import AsyncK8SManager
from polyaxon.utils.fqn_utils import get_resource_name_for_kind


@transaction.non_atomic_requests
async def k8s_inspect(
    request: ASGIRequest,
    namespace: str,
    owner: str,
    project: str,
    run_uuid: str,
    methods: Optional[Dict] = None,
) -> HttpResponse:
    validate_methods(request, methods)
    resource_name = get_resource_name_for_kind(run_uuid=run_uuid)
    k8s_manager = AsyncK8SManager(
        namespace=settings.CLIENT_CONFIG.namespace,
        in_cluster=settings.CLIENT_CONFIG.in_cluster,
    )
    await k8s_manager.setup()
    k8s_operation = await get_k8s_operation(
        k8s_manager=k8s_manager, resource_name=resource_name
    )
    data = None
    if k8s_operation:
        data = await get_pods(k8s_manager=k8s_manager, run_uuid=run_uuid)
    if k8s_manager:
        await k8s_manager.close()
    return UJSONResponse(data or {})


URLS_RUNS_K8S_INSPECT = (
    "<str:namespace>/<str:owner>/<str:project>/runs/<str:run_uuid>/k8s_inspect"
)

# fmt: off
k8s_routes = [
    path(
        URLS_RUNS_K8S_INSPECT,
        k8s_inspect,
        name="k8s_inspect",
        kwargs=dict(methods=["GET"]),
    ),
]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

RUN = "run"
ACTOR = "actor"
PROJECT = "project"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import factory

from haupt.db.abstracts.getter import get_project_model


class ProjectFactory(factory.django.DjangoModelFactory):
    name = factory.Sequence("project-{}".format)

    class Meta:
        model = get_project_model()



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
import os

from typing import Optional

from haupt.cli.runners.base import start_app
from polyaxon.env_vars.keys import EV_KEYS_PROXY_STREAMS_TARGET_PORT
from polyaxon.services.values import PolyaxonServices


def start(
    host: Optional[str] = None,
    port: Optional[int] = None,
    log_level: Optional[str] = None,
    workers: Optional[int] = None,
    per_core: bool = False,
    uds: Optional[str] = None,
):
    port = port or os.environ.get(EV_KEYS_PROXY_STREAMS_TARGET_PORT)
    start_app(
        app="haupt.polyconf.asgi.viewer:application",
        app_name=PolyaxonServices.STREAMS,
        host=host,
        port=port,
        log_level=log_level,
        workers=workers,
        per_core=per_core,
        uds=uds,
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

NAME = "haupt"
VERSION = "2.0.0-rc12"
DESC = (
    "Lineage metadata API, artifacts streams, sandbox, ML-API, and spaces for Polyaxon."
)
URL = "https://github.com/polyaxon/haupt"
AUTHOR = "Polyaxon, Inc."
EMAIL = "contact@polyaxon.com"
LICENSE = "AGPLv3"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from clipped.utils.tz import now
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView

from haupt import pkg
from haupt.common import conf
from haupt.common.options.registry.installation import ORGANIZATION_KEY
from haupt.db.managers.dummy_key import get_dummy_key
from polyaxon.cli.session import get_compatibility
from polyaxon.schemas.cli.cli_config import CliConfig
from polyaxon.services.values import PolyaxonServices


class HealthView(APIView):
    authentication_classes = ()
    throttle_scope = "checks"
    HEALTH_FILE = "/tmp/.compatibility"

    def get_config(self):
        try:
            return CliConfig.read(self.HEALTH_FILE, config_type=".json")
        except Exception:  # noqa
            return

    def get(self, request, *args, **kwargs):
        CliConfig.init_file(self.HEALTH_FILE)
        config = self.get_config()
        if config and config.should_check():
            config.version = pkg.VERSION
            key = conf.get(ORGANIZATION_KEY) or get_dummy_key()
            compatibility = get_compatibility(
                key=key,
                service=PolyaxonServices.PLATFORM,
                version=config.version,
                is_cli=False,
            )
            config.compatibility = compatibility.to_dict() if compatibility else None
            config.last_check = now()
            config.write(self.HEALTH_FILE)
        return Response(status=status.HTTP_200_OK)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.config_manager import ConfigManager


def set_core(context, config: ConfigManager, use_db: bool = True):
    context["DEBUG"] = config.is_debug_mode
    context["POLYAXON_SERVICE"] = config.service
    context["POLYAXON_ENVIRONMENT"] = config.env
    context["CHART_VERSION"] = config.chart_version
    context["SCHEDULER_ENABLED"] = config.scheduler_enabled
    context["K8S_NAMESPACE"] = config.namespace

    context["FILE_UPLOAD_PERMISSIONS"] = 0o644

    context["WSGI_APPLICATION"] = "{}.wsgi.application".format(config.config_module)
    context["TIME_ZONE"] = config.timezone
    context["LANGUAGE_CODE"] = "en"
    context["LANGUAGES"] = (("en", "English"),)

    context["USE_I18N"] = True
    context["USE_TZ"] = True

    context["INTERNAL_IPS"] = ("127.0.0.1",)
    context["APPEND_SLASH"] = True

    context["ROOT_URLCONF"] = ""

    if use_db:
        db_engine = (
            "django.db.backends.sqlite3"
            if config.is_sqlite_db_engine
            else "django.db.backends.postgresql"
        )
        context["AUTH_USER_MODEL"] = "db.User"
        context["DB_ENGINE_NAME"] = config.db_engine_name
        context["DEFAULT_DB_ENGINE"] = db_engine
        db_name = config.get("POLYAXON_DB_NAME", "str", is_optional=True)
        if not db_name:
            db_name = "/tmp/plxdb" if config.is_sqlite_db_engine else "polyaxon"
        db_definition = {
            "ENGINE": db_engine,
            "NAME": db_name,
            "USER": config.get("POLYAXON_DB_USER", "str", is_optional=True),
            "PASSWORD": config.get(
                "POLYAXON_DB_PASSWORD", "str", is_secret=True, is_optional=True
            ),
            "HOST": config.get("POLYAXON_DB_HOST", "str", is_optional=True),
            "PORT": config.get("POLYAXON_DB_PORT", "str", is_optional=True),
            "ATOMIC_REQUESTS": True,
            "CONN_MAX_AGE": config.get(
                "POLYAXON_DB_CONN_MAX_AGE",
                "int",
                is_optional=True,
                default=60,
            ),
        }
        db_options = config.get(
            "POLYAXON_DB_OPTIONS", "dict", is_optional=True, default={}
        )
        if db_options:
            db_definition["OPTIONS"] = db_options
        context["DATABASES"] = {"default": db_definition}



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.db.abstracts.artifacts import BaseArtifact, BaseArtifactLineage


class Artifact(BaseArtifact):
    class Meta:
        app_label = "db"
        unique_together = (("name", "state"),)
        db_table = "db_artifact"


class ArtifactLineage(BaseArtifactLineage):
    class Meta:
        app_label = "db"
        unique_together = (("run", "artifact", "is_input"),)
        db_table = "db_artifactlineage"



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from rest_framework.urlpatterns import format_suffix_patterns

from django.urls import re_path

from haupt.apis.projects import views
from haupt.common.apis.urls import projects

projects_urlpatterns = [
    re_path(projects.URLS_PROJECTS_CREATE, views.ProjectCreateView.as_view()),
    re_path(projects.URLS_PROJECTS_LIST, views.ProjectListView.as_view()),
    re_path(projects.URLS_PROJECTS_NAMES, views.ProjectNameListView.as_view()),
    re_path(projects.URLS_PROJECTS_DETAILS, views.ProjectDetailView.as_view()),
]

# Order is important, because the patterns could swallow other urls
urlpatterns = format_suffix_patterns(projects_urlpatterns)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.conf import settings
from django.contrib.postgres.fields import ArrayField
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models


class TagModel(models.Model):
    # Special handing for sqlite without migrating previous installation using ArrayField
    tags = (
        models.JSONField(
            encoder=DjangoJSONEncoder,
            blank=True,
            null=True,
        )
        if settings.DB_ENGINE_NAME == "sqlite"
        else ArrayField(
            base_field=models.CharField(max_length=64), blank=True, null=True
        )
    )

    class Meta:
        abstract = True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import re

import django.core.validators

from django.db import migrations, models

import haupt.common.validation.blacklist


class Migration(migrations.Migration):
    dependencies = [
        ("db", "0008_run_wait_time"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project",
            name="name",
            field=models.CharField(
                max_length=128,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        re.compile("^[-a-zA-Z0-9_]+\\Z"),
                        "Enter a valid “slug” consisting of letters, numbers, underscores or hyphens.",
                        "invalid",
                    ),
                    haupt.common.validation.blacklist.validate_blacklist_name,
                ],
            ),
        ),
    ]



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
import logging
import os

from typing import Optional

from clipped.utils.workers import get_core_workers

import uvicorn

from polyaxon.env_vars.keys import EV_KEYS_PROXY_LOCAL_PORT

_logger = logging.getLogger("haupt.cli")


def migrate(
    migrate_tables: bool = False,
    migrate_db: bool = False,
):
    from django.core.management import execute_from_command_line

    # Required env var to trigger a management command
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "haupt.polyconf.settings")

    if migrate_tables:
        argv = ["manage.py", "tables"]
        _logger.info("Starting tables migration ...")
        execute_from_command_line(argv)
        _logger.info("Tables were migrated correctly!")

    if migrate_db:
        argv = ["manage.py", "migrate"]
        execute_from_command_line(argv)
        _logger.info("DB Migration finished!")


def start_app(
    app,
    app_name,
    host: Optional[str] = None,
    port: Optional[int] = None,
    log_level: Optional[str] = None,
    workers: Optional[int] = None,
    per_core: bool = False,
    max_workers: Optional[int] = None,
    uds: Optional[str] = None,
    migrate_tables: bool = False,
    migrate_db: bool = False,
):
    migrate(migrate_tables=migrate_tables, migrate_db=migrate_db)
    host = host or "0.0.0.0"
    port = int(port or 8000)
    os.environ[EV_KEYS_PROXY_LOCAL_PORT] = str(port)
    log_level = log_level or "warning"
    if per_core:
        workers = get_core_workers(per_core=workers or 2, max_workers=max_workers)
    else:
        workers = workers or get_core_workers(per_core=2, max_workers=max_workers)

        _logger.info(
            "{app_name} is running on http://{host}:{port} in process {pid}".format(
                app_name=app_name, host=host, port=port, pid=os.getpid()
            )
        )
    uvicorn.run(
        app,
        host=host,
        port=port,
        access_log=False,
        log_level=log_level.lower(),
        workers=workers,
        uds=uds,
        timeout_keep_alive=120,
    )



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.conf import settings

from haupt.common.service_interface import LazyServiceWrapper
from haupt.orchestration.operations.service import OperationInitSpec, OperationsService


def get_operation_backend_path():
    return (
        settings.OPERATIONS_BACKEND
        or "haupt.orchestration.operations.service.OperationsService"
    )


backend = LazyServiceWrapper(
    backend_base=OperationsService,
    backend_path=get_operation_backend_path(),
    options={},
)
backend.expose(locals())



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from django.contrib.admin import site
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin

from haupt.db.abstracts.getter import (
    get_artifact_model,
    get_project_model,
    get_run_model,
)
from haupt.db.administration.artifacts import ArtifactAdmin
from haupt.db.administration.projects import ProjectAdmin
from haupt.db.administration.runs import RunLightAdmin

site.register(get_user_model(), UserAdmin)
site.register(get_artifact_model(), ArtifactAdmin)
site.register(get_project_model(), ProjectAdmin)
site.register(get_run_model(), RunLightAdmin)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
import os

from clipped.utils.json import orjson_loads
from clipped.utils.paths import check_or_create_path, delete_path, untar_file
from rest_framework import status

from django.core.handlers.asgi import ASGIRequest
from django.http import HttpResponse

from asgiref.sync import sync_to_async
from polyaxon import settings
from polyaxon.fs.async_manager import upload_dir, upload_file
from polyaxon.fs.types import FSSystem
from polyaxon.lifecycle import V1ProjectFeature


async def handle_posted_data(
    fs: FSSystem,
    content_file: any,
    root_path: str,
    path: str,
    upload: bool,
    is_file: bool,
    overwrite: bool = True,
    untar: bool = True,
) -> str:
    filename = content_file.name
    tmp_path = "{}/{}".format(root_path, os.path.basename(filename)).rstrip("/")
    if path:
        root_path = "{}/{}".format(root_path, path).rstrip("/")
        if is_file:
            root_path = "{}/{}".format(root_path, os.path.basename(filename))
    else:
        if untar:
            root_path = "{}/{}".format(root_path, path or "/").rstrip("/")
        else:
            root_path = tmp_path
    if not untar:
        tmp_path = root_path

    full_tmppath = settings.AGENT_CONFIG.get_local_path(
        subpath=tmp_path, entity=V1ProjectFeature.RUNTIME
    )
    full_filepath = settings.AGENT_CONFIG.get_local_path(
        subpath=root_path, entity=V1ProjectFeature.RUNTIME
    )
    store_path = settings.AGENT_CONFIG.get_store_path(
        subpath=root_path, entity=V1ProjectFeature.RUNTIME
    )

    if overwrite and os.path.exists(full_filepath):
        delete_path(full_filepath)
    if not overwrite and os.path.exists(full_filepath):
        return full_filepath
    # Always clean tmp path
    if overwrite and os.path.exists(full_tmppath):
        delete_path(full_tmppath)

    check_or_create_path(full_tmppath, is_dir=False)
    check_or_create_path(full_filepath, is_dir=not is_file)

    # Creating the new file
    with open(full_tmppath, "wb") as destination:
        for chunk in content_file.chunks():
            destination.write(chunk)
    if untar:
        await sync_to_async(untar_file)(
            full_tmppath, extract_path=full_filepath, use_filepath=False
        )
    if upload and store_path != full_filepath:
        if is_file:
            await upload_file(fs=fs, subpath=root_path)
        else:
            await upload_dir(fs=fs, subpath=root_path)
    return root_path


async def handle_upload(
    fs: FSSystem, request: ASGIRequest, run_uuid: str, is_file: bool
) -> HttpResponse:
    content_file = request.FILES["upload_file"]
    content_json = request.POST.get("json")
    content_json = orjson_loads(content_json) if content_json else {}
    overwrite = content_json.get("overwrite", True)
    untar = content_json.get("untar", True)
    path = content_json.get("path", "")
    try:
        archived_path = await handle_posted_data(
            fs=fs,
            content_file=content_file,
            root_path=run_uuid,
            path=path,
            upload=True,
            is_file=is_file,
            overwrite=overwrite,
            untar=untar,
        )
    except Exception as e:
        return HttpResponse(
            content="Run's artifacts upload was unsuccessful, "
            "an error was raised while uploading the data %s." % e,
            status=status.HTTP_400_BAD_REQUEST,
        )
    if not archived_path:
        return HttpResponse(
            content="Artifact not found and not uploaded: filepath={}".format(
                archived_path
            ),
            status=status.HTTP_404_NOT_FOUND,
        )
    return HttpResponse(status=status.HTTP_200_OK)



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.
from typing import Optional, Tuple

from rest_framework import HTTP_HEADER_ENCODING, exceptions
from rest_framework.authentication import get_authorization_header

from django.http import HttpRequest

from haupt.common.headers import get_internal_header
from polyaxon.services.auth import AuthenticationTypes
from polyaxon.services.values import PolyaxonServices


def get_internal_auth(
    request: HttpRequest, keyword: str = AuthenticationTypes.INTERNAL_TOKEN
) -> Optional[Tuple[str, str]]:
    auth = get_authorization_header(request).split()
    internal_service = get_internal_header(request)
    try:
        internal_service = internal_service.decode()
    except UnicodeError:
        msg = (
            "Invalid internal_service header. "
            "internal_service string should not contain invalid characters."
        )
        raise exceptions.AuthenticationFailed(msg)

    if internal_service not in PolyaxonServices.to_set():
        return None

    if not auth or auth[0].lower() != keyword.lower().encode(HTTP_HEADER_ENCODING):
        return None

    if len(auth) == 1:
        msg = "Invalid token header. No credentials provided."
        raise exceptions.AuthenticationFailed(msg)

    elif len(auth) > 2:
        msg = "Invalid token header. Token string should not contain spaces."
        raise exceptions.AuthenticationFailed(msg)

    try:
        token = auth[1].decode()
    except UnicodeError:
        msg = (
            "Invalid token header. Token string should not contain invalid characters."
        )
        raise exceptions.AuthenticationFailed(msg)

    return internal_service, token



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

import logging
import traceback

from typing import Dict, List, Optional

from django.utils.timezone import now

from haupt.common import conf
from haupt.common.options.registry.k8s import K8S_IN_CLUSTER, K8S_NAMESPACE
from haupt.db.abstracts.getter import get_run_model
from haupt.db.abstracts.runs import BaseRun
from haupt.db.managers.artifacts import atomic_set_artifacts
from haupt.db.managers.statuses import new_run_status, new_run_stop_status
from haupt.orchestration.scheduler import resolver
from kubernetes.client.rest import ApiException
from polyaxon import operations, settings
from polyaxon.agents import manager
from polyaxon.auxiliaries.default_scheduling import V1DefaultScheduling
from polyaxon.exceptions import (
    PolyaxonCompilerError,
    PolyaxonK8SError,
    PolypodException,
)
from polyaxon.lifecycle import LifeCycle, V1StatusCondition, V1Statuses
from traceml.artifacts import V1RunArtifact

_logger = logging.getLogger("polyaxon.scheduler")


def get_run(
    run_id: int,
    run: Optional[BaseRun],
    use_all: bool = False,
    only: Optional[List[str]] = None,
    defer: Optional[List[str]] = None,
    prefetch: Optional[List[str]] = None,
) -> Optional[BaseRun]:
    if run:
        return run
    run_model = get_run_model()
    query = run_model.all if use_all else run_model.objects
    if only:
        query = query.only(*only)
    if defer:
        query = query.only(*defer)
    if prefetch:
        query = query.select_related(*prefetch)

    try:
        return query.get(id=run_id)
    except run_model.DoesNotExist:
        _logger.info(
            "Something went wrong, the run `%s` does not exist anymore.", run_id
        )


def runs_artifacts_clean(run: BaseRun):
    if run.is_managed:
        in_cluster = conf.get(K8S_IN_CLUSTER)
        if in_cluster and (run.is_service or run.is_job):
            op = operations.get_cleaner_operation(
                connection=settings.AGENT_CONFIG.artifacts_store,
                run_uuid=run.uuid.hex,
                run_kind=run.kind,
                environment=V1DefaultScheduling.get_service_environment(
                    settings.AGENT_CONFIG.cleaner,
                    settings.AGENT_CONFIG.default_scheduling,
                ),
                cleaner=settings.AGENT_CONFIG.cleaner,
            )
            try:
                manager.make_and_create(
                    content=op,
                    owner_name=run.project.owner.name,
                    project_name=run.project.name,
                    run_name=run.name,
                    run_uuid=run.uuid.hex,
                    run_kind=run.kind,
                    namespace=conf.get(K8S_NAMESPACE),
                    in_cluster=in_cluster,
                )
            except Exception as e:
                _logger.warning(
                    "Failed to run cleaning job for %s.%s.%s.\n %s",
                    run.project.owner.name,
                    run.project.name,
                    run.uuid.hex,
                    e,
                )


def runs_delete(run_id: int, run: Optional[BaseRun]):
    run = get_run(run_id=run_id, run=run, use_all=True)
    if not run:
        return

    runs_stop(run_id=run_id, run=run, update_status=False, clean=True)
    runs_artifacts_clean(run)
    run.delete()


def runs_prepare(
    run_id: int,
    run: Optional[BaseRun],
    eager: bool = False,
    extra_message: Optional[str] = None,
) -> bool:
    run = get_run(run_id=run_id, run=run, prefetch=["project"])
    if not run:
        return False

    if not LifeCycle.is_compilable(run.status):
        _logger.info(
            "Run `%s` cannot transition from `%s` to `%s`.",
            run_id,
            run.status,
            V1Statuses.COMPILED,
        )
        return False

    try:
        compiled_at = now()
        _, compiled_operation = resolver.resolve(
            run=run, compiled_at=compiled_at, eager=eager
        )
    except PolyaxonCompilerError as e:
        condition = V1StatusCondition.get_condition(
            type=V1Statuses.FAILED,
            status="True",
            reason="SchedulerPrepare",
            message=f"Failed to compile.\n{e}",
        )
        new_run_status(run=run, condition=condition)
        return False
    except Exception as e:
        condition = V1StatusCondition.get_condition(
            type=V1Statuses.FAILED,
            status="True",
            reason="SchedulerPrepare",
            message=f"Compiler received an internal error.\n{e}",
        )
        new_run_status(run=run, condition=condition)
        return False

    message = "Run is compiled"
    if extra_message:
        message = f"{message} ({extra_message})."
    condition = V1StatusCondition.get_condition(
        type=V1Statuses.COMPILED,
        status="True",
        reason="SchedulerPrepare",
        message=message,
        last_update_time=compiled_at,
    )
    new_run_status(run=run, condition=condition)

    if run.pending:
        return False

    if eager:
        runs_start(run_id=run.id, run=run)
        return False

    return True


def runs_start(run_id: int, run: Optional[BaseRun]):
    run = get_run(run_id=run_id, run=run)
    if not run:
        return

    if not run.is_managed:
        return

    if not LifeCycle.is_compiled(run.status):
        _logger.info(
            "Run `%s` cannot transition from `%s` to `%s`.",
            run_id,
            run.status,
            V1Statuses.QUEUED,
        )
        return

    condition = V1StatusCondition.get_condition(
        type=V1Statuses.QUEUED,
        status="True",
        reason="SchedulerStart",
        message="Run is queued",
    )
    new_run_status(run=run, condition=condition)

    def _log_error(exc: Exception, message: Optional[str] = None):
        message = message or "Could not start the operation.\n"
        message += "error: {}\n{}".format(repr(exc), traceback.format_exc())
        cond = V1StatusCondition.get_condition(
            type=V1Statuses.FAILED,
            status="True",
            reason="SchedulerStart",
            message=message,
        )
        new_run_status(run=run, condition=cond)

    try:
        in_cluster = conf.get(K8S_IN_CLUSTER)
        if in_cluster and (run.is_service or run.is_job):
            manager.start(
                content=run.content,
                owner_name=run.project.owner.name,
                project_name=run.project.name,
                run_name=run.name,
                run_uuid=run.uuid.hex,
                run_kind=run.kind,
                namespace=conf.get(K8S_NAMESPACE),
                in_cluster=in_cluster,
                default_auth=False,
            )
        return
    except (PolyaxonK8SError, ApiException) as e:
        _log_error(exc=e, message="Kubernetes manager could not start the operation.\n")
    except PolypodException as e:
        _log_error(exc=e, message="Failed converting the run manifest.\n")
    except Exception as e:
        _log_error(exc=e, message="Failed with unknown exception.\n")


def runs_set_artifacts(run_id: int, run: Optional[BaseRun], artifacts: List[Dict]):
    run = get_run(run_id=run_id, run=run)
    if not run:
        return

    artifacts = [V1RunArtifact.from_dict(a) for a in artifacts]
    atomic_set_artifacts(run=run, artifacts=artifacts)


def runs_stop(
    run_id: int,
    run: Optional[BaseRun],
    update_status=False,
    message=None,
    clean=False,
) -> bool:
    run = get_run(run_id=run_id, run=run)
    if not run:
        return True

    should_stop = (
        LifeCycle.is_k8s_stoppable(run.status) or run.status == V1Statuses.STOPPING
    )

    def _clean():
        try:
            manager.clean(
                run_uuid=run.uuid.hex,
                run_kind=run.kind,
                namespace=conf.get(K8S_NAMESPACE),
                in_cluster=in_cluster,
            )
        except (PolyaxonK8SError, ApiException) as e:
            _logger.warning(
                "Something went wrong, the run `%s` could not be stopped, error %s",
                run.uuid,
                e,
            )
            return False

    if run.is_managed and should_stop:
        in_cluster = conf.get(K8S_IN_CLUSTER)
        if in_cluster and (run.is_service or run.is_job):
            if clean:
                _clean()
            try:
                manager.stop(
                    run_uuid=run.uuid.hex,
                    run_kind=run.kind,
                    namespace=conf.get(K8S_NAMESPACE),
                    in_cluster=in_cluster,
                )
            except (PolyaxonK8SError, ApiException) as e:
                _logger.warning(
                    "Something went wrong, the run `%s` could not be stopped, error %s",
                    run.uuid,
                    e,
                )
                return False

    if not update_status:
        return True

    new_run_stop_status(run=run, message=message)
    return True



#!/usr/bin/python
#
# Copyright 2018-2023 Polyaxon, Inc.
# This file and its contents are licensed under the AGPLv3 License.
# Please see the included NOTICE for copyright information and
# LICENSE-AGPL for a copy of the license.

from haupt.common.config_manager import ConfigManager
from polyaxon.api import STATIC_V1
from polyaxon.env_vars.keys import (
    EV_KEYS_ARTIFACTS_ROOT,
    EV_KEYS_STATIC_ROOT,
    EV_KEYS_STATIC_URL,
)


def set_assets(context, config: ConfigManager):
    context["MEDIA_ROOT"] = config.get(
        "POLYAXON_MEDIA_ROOT", "str", is_optional=True, default=""
    )
    context["MEDIA_URL"] = config.get(
        "POLYAXON_MEDIA_URL", "str", is_optional=True, default=""
    )

    context["STATIC_ROOT"] = config.get(
        EV_KEYS_STATIC_ROOT,
        "str",
        is_optional=True,
        default=str(config.root_dir / "static"),
    )
    context["STATIC_URL"] = config.get(
        EV_KEYS_STATIC_URL,
        "str",
        is_optional=True,
        default="/static/",
    )

    # Additional locations of static files
    context["STATICFILES_DIRS"] = (str(config.root_dir / "public"),)

    context["STATICFILES_FINDERS"] = (
        "django.contrib.staticfiles.finders.FileSystemFinder",
        "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    )

    context["LOCALE_PATHS"] = (
        str(config.root_dir / "locale"),
        str(config.root_dir / "client" / "js" / "libs" / "locale"),
    )

    context["STATICI18N_ROOT"] = STATIC_V1
    context["STATICI18N_OUTPUT_DIR"] = "jsi18n"

    context["ARTIFACTS_ROOT"] = config.get(
        EV_KEYS_ARTIFACTS_ROOT,
        "str",
        is_optional=True,
        default="/tmp/plx/artifacts_uploads",
    )
    context["ARCHIVES_ROOT"] = config.get(
        "POLYAXON_ARCHIVES_ROOT",
        "str",
        is_optional=True,
        default="/tmp/plx/archives",
    )
